# -*- coding: UTF-8 -*-
"""
Name: io_tools.py
Porpose: input/output redirection to processes (aka threads)
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.17.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import requests
import wx
from videomass.vdms_threads.ffplay_file import FilePlay
from videomass.vdms_threads import generic_downloads
from videomass.vdms_threads.volumedetect import VolumeDetectThread
from videomass.vdms_threads.check_bin import (ff_conf,
                                              ff_formats,
                                              ff_codecs,
                                              ff_topics,
                                              )
from videomass.vdms_utils.utils import open_default_application
from videomass.vdms_dialogs.widget_utils import PopupDialog
from videomass.vdms_ytdlp.ydl_extractinfo import YdlExtractInfo


def youtubedl_getstatistics(url, ssl, parent=None):
    """
    Call `YdlExtractInfo` thread to extract data info.
    During this process a wait pop-up dialog is shown.

    Returns a generator.

    Usage example without pop-up dialog:
        thread = YdlExtractInfo(url)
        thread.join()
        data = thread.data
        yield data
    """
    thread = YdlExtractInfo(url, ssl)
    dlgload = PopupDialog(parent,
                          _("Videomass - Loading..."),
                          _("Wait....\nRetrieving required data."))
    dlgload.ShowModal()
    # thread.join()
    data = thread.data
    dlgload.Destroy()
    yield data
# --------------------------------------------------------------------------#


def stream_play(filepath, timeseq, param, autoexit):
    """
    Call Thread for playback with ffplay
    """
    get = wx.GetApp()  # get data from bootstrap

    if timeseq:
        splseq = timeseq.split()
        tseq = f'{splseq[0]} {splseq[1]}', f'{splseq[2]} {splseq[3]}'
    else:
        tseq = '', ''
    try:
        with open(filepath, encoding='utf8'):
            FilePlay(filepath,
                     tseq,
                     param,
                     get.appset['logdir'],
                     get.appset['ffplay_cmd'],
                     get.appset['ffplay_default_args'],
                     autoexit
                     )
            # thread.join() > attende fine thread, se no ritorna subito
            # error = thread.data
    except IOError:
        wx.MessageBox(_("File does not exist or is invalid:  %s") % (
            filepath), "Videomass", wx.ICON_EXCLAMATION, None)
        return
# -----------------------------------------------------------------------#


def volume_detect_process(filelist, timeseq, audiomap, parent=None):
    """
    Run thread to get audio peak level data and show a
    pop-up dialog with message.
    """
    get = wx.GetApp()

    if timeseq:
        splseq = timeseq.split()
        tseq = f'{splseq[0]} {splseq[1]}', f'{splseq[2]} {splseq[3]}'
    else:
        tseq = '', ''
    thread = VolumeDetectThread(tseq,
                                filelist,
                                audiomap,
                                get.appset['logdir'],
                                get.appset['ffmpeg_cmd']
                                )
    dlgload = PopupDialog(parent,
                          _("Videomass - Loading..."),
                          _("Wait....\nAudio peak analysis."))
    dlgload.ShowModal()
    # thread.join()
    data = thread.data
    dlgload.Destroy()

    return data
# -------------------------------------------------------------------------#


def test_conf():
    """
    Call `check_bin.ffmpeg_conf` to get data to test the building
    configurations of the used FFmpeg executable.
    """
    get = wx.GetApp()
    out = ff_conf(get.appset['ffmpeg_cmd'], get.appset['ostype'])
    return out
# -------------------------------------------------------------------------#


def test_formats():
    """
    Call `check_bin.ff_formats` to get available formats by
    FFmpeg executable.
    """
    get = wx.GetApp()
    out = ff_formats(get.appset['ffmpeg_cmd'], get.appset['ostype'])
    return out
# -------------------------------------------------------------------------#


def test_codecs(type_opt):
    """
    Call `check_bin.ff_codecs` to get available encoders
    and decoders by FFmpeg executable.
    """
    get = wx.GetApp()
    out = ff_codecs(get.appset['ffmpeg_cmd'],
                    type_opt,
                    get.appset['ostype'],
                    )
    return out
# -------------------------------------------------------------------------#


def findtopic(topic):
    """
    Call * check_bin.ff_topic * to run the ffmpeg command to search
    a certain topic..
    """
    get = wx.GetApp()
    retcod = ff_topics(get.appset['ffmpeg_cmd'], topic, get.appset['ostype'])

    if 'Not found' in retcod[0]:
        notf = f"\n{retcod[1]}"
        return notf

    return retcod[1]
# -------------------------------------------------------------------------#


def openpath(where):
    """
    Call `vdms_threads.opendir.open_default_application`.
    """
    ret = open_default_application(where)
    if ret:
        wx.MessageBox(ret, 'Videomass', wx.ICON_ERROR, None)
# -------------------------------------------------------------------------#


def get_github_releases(url, keyname):
    """
    Check for releases data on github page using github API:
        https://developer.github.com/v3/repos/releases/#get-the-latest-release

    see keyname examples here:
    <https://api.github.com/repos/jeanslack/Videomass/releases>
    """
    try:
        response = requests.get(url, timeout=15)
        not_found = None, None

    except Exception as err:
        not_found = 'request error:', err

    else:

        try:
            version = response.json()[f"{keyname}"]

        except Exception as err:
            not_found = 'response error:', err

    if not_found[0]:
        return not_found

    return version, None
# --------------------------------------------------------------------------#


def get_presets(url, dest, msg, parent=None):
    """
    get latest Videomass presets
    """
    thread = generic_downloads.FileDownloading(url, dest)
    dlgload = PopupDialog(parent, _("Videomass - Downloading..."), msg)
    dlgload.ShowModal()
    # thread.join()
    status = thread.data
    dlgload.Destroy()

    return status
# --------------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
Name: filedrop.py
Porpose: files drag n drop interface
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.18.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import re
import wx
from pubsub import pub
from videomass.vdms_io.io_tools import stream_play
from videomass.vdms_threads.ffprobe import ffprobe
from videomass.vdms_utils.utils import time_to_integer
from videomass.vdms_utils.utils import to_bytes
from videomass.vdms_dialogs.renamer import Renamer
from videomass.vdms_dialogs.list_warning import ListWarning


def fullpathname_sanitize(fullpathfilename):
    """
    Check for 'full path file name' sanitize.
    return message `string` if warning, `None` otherwise
    """
    illegal = '^` ~ " # % & * : < > ? / \\ { | }'
    illegalchars = _("File has illegal characters like:")
    invalidpath = _("Invalid character found in path name: (\")")
    justfile = _("Directories are not allowed, just add files, please.")
    noext = _("File without format extension: please give an "
              "appropriate extension to the file name, example "
              "'.mkv', '.avi', '.mp3', etc.")

    check = bool(re.search(r"^(?:[^\^\`\~\"\#\%\&\*\:\<\>\?\/\\\{\|\}])*$",
                 os.path.basename(fullpathfilename)))
    if check is not True:
        return f'{illegalchars} {illegal}'

    if '"' in fullpathfilename:
        return invalidpath

    if os.path.isdir(fullpathfilename):
        return justfile

    if os.path.splitext(os.path.basename(fullpathfilename))[1] == '':
        return noext

    return None
# ----------------------------------------------------------------------#


def filename_sanitize(newname, outputnames):
    """
    Check for 'filename' sanitize. It performs a check to
    ensure that output files do NOT have the same name,
    or illegal chars like:

            ^` ~ " # % & * : < > ? / \\ { | }

    The maximum filename lengh is fixed to 255 characters.

    return a str(string) if warning,
    return None otherwise
    """
    illegal = '^` ~ " # % & * : < > ? / \\ { | }'
    msg_invalid = _('Name has illegal characters like: {0}').format(illegal)
    msg_inuse = _('Name already in use:')

    check = bool(re.search(r"^(?:[^\^\`\~\"\#\%\&\*\:\<\>\?\/\\\{\|\}]"
                           r"{1,255}$)*$", newname))
    if check is not True:
        return f'{msg_invalid}'

    if newname in outputnames:
        return f'{msg_inuse} {newname}'

    return None
# ----------------------------------------------------------------------


class MyListCtrl(wx.ListCtrl):
    """
    This is the listControl widget.
    Note that this wideget has DnDPanel parented.
    """
    def __init__(self, parent):
        """
        Constructor.
        WARNING to avoid segmentation error on removing items by
        listctrl, style must be wx.LC_SINGLE_SEL .
        """
        get = wx.GetApp()
        self.ffprobe_cmd = get.appset['ffprobe_cmd']
        self.index = None
        self.parent = parent  # parent is DnDPanel class
        self.data = self.parent.data
        self.file_src = self.parent.file_src
        self.duration = self.parent.duration
        self.outputnames = self.parent.outputnames
        self.errors = {}
        wx.ListCtrl.__init__(self,
                             parent,
                             style=wx.LC_REPORT
                             | wx.LC_SINGLE_SEL,
                             )
    # ----------------------------------------------------------------------#

    def dropUpdate(self, path, newname=None):
        """
        Update list-control during drag and drop.
        Note that the optional 'newname' argument is given by
        the 'on_col_click' method in the 'FileDnD' class to preserve
        the related renames in column 5 of wx.ListCtrl.

        """
        self.index = self.GetItemCount()
        warn = fullpathname_sanitize(path)  # check for fullname sanitize
        if warn:
            self.errors[f'"{path}"'] = warn
            return

        if not [x for x in self.data if x['format']['filename'] == path]:
            probe = ffprobe(path, self.ffprobe_cmd,
                            hide_banner=None, pretty=None)
            if probe[1]:
                self.errors[f'"{path}"'] = probe[1]
                return

            probe = probe[0]
            self.InsertItem(self.index, str(self.index + 1))
            self.SetItem(self.index, 1, path)

            if 'duration' not in probe['format'].keys():
                self.SetItem(self.index, 2, 'N/A')
                # NOTE these are my custom adds to probe data
                probe['format']['time'] = '00:00:00.000'
                probe['format']['duration'] = 0

            else:
                tdur = probe['format']['duration'].split(':')
                sec, msec = tdur[2].split('.')[0], tdur[2].split('.')[1]
                tdur = f'{tdur[0]}h : {tdur[1]}m : {sec} : {msec}'
                self.SetItem(self.index, 2, tdur)
                probe['format']['time'] = probe.get('format').pop('duration')
                time = time_to_integer(probe.get('format')['time'])
                probe['format']['duration'] = time

            media = probe['streams'][0]['codec_type']
            formatname = probe['format']['format_long_name']
            self.SetItem(self.index, 3, f'{media}: {formatname}')
            self.SetItem(self.index, 4, probe['format']['size'])
            if newname:
                self.SetItem(self.index, 5, newname)
                self.outputnames.append(newname)
            else:
                fname = os.path.splitext(os.path.basename(path))[0]
                self.SetItem(self.index, 5, fname)
                self.outputnames.append(fname)
            self.index += 1
            self.data.append(probe)
            self.file_src.append(path)
            self.duration.append(probe['format']['duration'])
            # self.parent.statusbar_msg('', None)
            self.parent.changes_in_progress()
        else:
            mess = _("Duplicate file, it has already been added to the list.")
            self.errors[f'"{path}"'] = mess
    # ----------------------------------------------------------------------#

    def rejected_files(self):
        """
        Handles all rejected files if any
        """
        if self.errors:
            msg = _('Error details')
            with ListWarning(self,
                             self.errors,
                             caption=_('Rejected Files'),
                             header=msg,
                             buttons='OK',
                             ) as log:
                log.ShowModal()
            self.errors.clear()  # clear values here


class FileDrop(wx.FileDropTarget):
    """
    This is the file drop target class
    """
    # ----------------------------------------------------------------------#

    def __init__(self, window):
        """
        Constructor. File Drop targets are subsets of windows
        """
        wx.FileDropTarget.__init__(self)
        self.window = window  # window is MyListCtr class
    # ----------------------------------------------------------------------#

    def OnDropFiles(self, x, y, filenames):
        """
        When files are dropped, write where they were dropped and then
        the file paths themselves
        """
        for filepath in filenames:
            self.window.dropUpdate(filepath)  # update list control
        self.window.rejected_files()  # call parent.rejected_files at end

        return True
    # ----------------------------------------------------------------------#


class FileDnD(wx.Panel):
    """
    This panel hosts a listctrl for Drag AND Drop actions on which
    you can add new files, select them, remove them, play them,
    sort them or remove them from the listctrl.

    """
    # CONSTANTS:
    YELLOW = '#bd9f00'
    BLACK = '#060505'  # black for background status bar
    ORANGE = '#f28924'

    def __init__(self, parent, *args):
        """Constructor. This will initiate with an id and a title"""
        get = wx.GetApp()
        appdata = get.appset
        self.themecolor = appdata['icontheme'][1]
        self.parent = parent  # parent is the MainFrame
        self.outputnames = args[1]
        self.data = args[2]
        self.file_src = args[3]
        self.duration = args[4]
        self.sortingstate = None  # ascending or descending order

        wx.Panel.__init__(self, parent, -1)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0, 25))
        # This builds the list control box:
        self.flCtrl = MyListCtrl(self)  # class MyListCtr
        # Establish the listctrl as a drop target:
        file_drop_target = FileDrop(self.flCtrl)
        self.flCtrl.SetDropTarget(file_drop_target)  # Make drop target.
        # populate columns
        colw = appdata['filedrop_column_width']
        self.flCtrl.InsertColumn(0, '#', width=colw[0])
        self.flCtrl.InsertColumn(1, _('File Name'), width=colw[1])
        self.flCtrl.InsertColumn(2, _('Duration'), width=colw[2])
        self.flCtrl.InsertColumn(3, _('Media type'), width=colw[3])
        self.flCtrl.InsertColumn(4, _('Size'), width=colw[4])
        self.flCtrl.InsertColumn(5, _('Output file name'), width=colw[5])
        # create widgets
        infomsg = _("Drag one or more files below")
        self.lbl_info = wx.StaticText(self, wx.ID_ANY, label=infomsg)
        sizer.Add(self.lbl_info, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add((0, 10))
        sizer.Add(self.flCtrl, 1, wx.EXPAND | wx.ALL, 2)
        sizer.Add((0, 10))
        sizer_outdir = wx.BoxSizer(wx.HORIZONTAL)
        self.btn_destpath = wx.Button(self, wx.ID_OPEN, "...", size=(35, -1),
                                      name='button destpath filedrop')
        self.text_path_save = wx.TextCtrl(self, wx.ID_ANY, "",
                                          style=wx.TE_PROCESS_ENTER
                                          | wx.TE_READONLY,
                                          )
        sizer_outdir.Add(self.text_path_save, 1, wx.ALL | wx.EXPAND, 2)
        sizer_outdir.Add(self.btn_destpath, 0, wx.ALL
                         | wx.ALIGN_CENTER_HORIZONTAL
                         | wx.ALIGN_CENTER_VERTICAL, 2,
                         )
        sizer.Add(sizer_outdir, 0, wx.EXPAND)
        self.SetSizer(sizer)

        if appdata['ostype'] == 'Darwin':
            self.lbl_info.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            self.lbl_info.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD))

        self.text_path_save.SetValue(args[0])
        if appdata['outputdir_asinput']:
            self.btn_destpath.Disable()
            self.text_path_save.Disable()

        # Tooltips
        self.btn_destpath.SetToolTip(_('Set a new destination for encodings'))
        self.text_path_save.SetToolTip(_("Current destination directory"))

        # Binding (EVT)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.flCtrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect, self.flCtrl)
        self.Bind(wx.EVT_LIST_COL_CLICK, self.on_col_click, self.flCtrl)

        pub.subscribe(self.text_information, "SET_DRAG_AND_DROP_TOPIC")
    # ----------------------------------------------------------------------

    def text_information(self, topic):
        """
        Set informative properties on selected topic.
        This method is called using pub/sub protocol
        "SET_DRAG_AND_DROP_TOPIC".
        """
        if topic in ('Presets Manager', 'Audio/Video Conversions',
                     'Concatenate Demuxer'):
            self.lbl_info.SetLabel(_('Drag one or more files below'))

        elif topic == 'Image Sequence to Video':
            self.lbl_info.SetLabel(_('Drag images below'))

        elif topic == 'Video to Pictures':
            self.lbl_info.SetLabel(_('Drag one or more video below'))
    # ----------------------------------------------------------------------

    def on_col_click(self, event):
        """
        Sort items by LEFT clicking on column headers
        (from ascending to descending and back to ascending).
        For this feature is required to delete all items from
        listctrl and data list before re-loading the same
        items with the new sorted order using `dropUpdate` method.

        if plane to use wx.EVT_LIST_COL_RIGHT_CLICK event:
            `if event.GetEventType() == wx.EVT_LIST_COL_RIGHT_CLICK.typeId:`
                `curritems.reverse()`
        see: <https://discuss.wxpython.org/t/event-geteventtype/22860/4>
        """
        count = self.flCtrl.GetItemCount()
        curritems = []
        if count > 1:
            if event.GetColumn() in (0, -1):
                return

            for x in range(count):
                curritems.append((self.flCtrl.GetItemText(x, col=1),
                                  self.flCtrl.GetItemText(x, col=2),
                                  self.flCtrl.GetItemText(x, col=3),
                                  self.flCtrl.GetItemText(x, col=4),
                                  self.flCtrl.GetItemText(x, col=5),
                                  ))
            if event.GetColumn() == 1:
                curritems.sort(key=lambda item: item[0])
            elif event.GetColumn() == 2:
                curritems.sort(key=lambda item: item[1])
            elif event.GetColumn() == 3:
                curritems.sort(key=lambda item: item[2])
            elif event.GetColumn() == 4:
                curritems.sort(key=lambda item:
                               to_bytes(''.join(item[3].split()), 'ffmpeg'))
            elif event.GetColumn() == 5:
                curritems.sort(key=lambda item: item[4])

            self.delete_all(None, setstate=False)  # does not setstate here

            if self.sortingstate == 'descending':
                self.sortingstate = 'ascending'
            elif self.sortingstate == 'ascending':
                self.sortingstate = 'descending'
            elif not self.sortingstate:
                self.sortingstate = 'ascending'

            if self.sortingstate == 'descending':
                curritems.reverse()

            for data in curritems:
                self.flCtrl.dropUpdate(data[0], data[4])
    # ----------------------------------------------------------------------

    def changes_in_progress(self, setfocus=True):
        """
        Routine operations on changes: receiving new files by
        opening, drag&drop, or removing files or sorting items by
        LEFT clicking on column headers in the ListCtrl.
        """
        self.parent.destroy_orphaned_window()
        self.parent.toolbar.EnableTool(9, True)
        if len(self.outputnames) > 1:
            self.parent.rename_batch.Enable(True)
        else:
            self.parent.rename_batch.Enable(False)

        if setfocus:
            sel = self.flCtrl.GetFocusedItem()  # Get the current row
            selitem = sel if sel != -1 else 0
            self.flCtrl.Focus(selitem)  # make the line the current line
            self.flCtrl.Select(selitem, on=1)  # default event selection
    # ----------------------------------------------------------------------

    def on_play_select(self, event):
        """
        Playback the selected file
        """
        if self.flCtrl.GetFirstSelected() == -1:  # None
            msg = _("No file selected")
            self.parent.statusbar_msg(msg, FileDnD.YELLOW, FileDnD.BLACK)
            return
        index = self.flCtrl.GetFocusedItem()
        item = self.flCtrl.GetItemText(index, 1)
        if self.parent.checktimestamp:
            tstamp = f'-vf "{self.parent.cmdtimestamp}"'
        else:
            tstamp = ""
        stream_play(item, self.parent.time_seq,
                    tstamp, self.parent.autoexit)
    # ----------------------------------------------------------------------

    def on_delete_selected(self, event):
        """
        Delete a selected file, if nothing is selected return None
        """
        if self.flCtrl.GetFirstSelected() == -1:  # None
            return

        item, indexes = -1, []
        while 1:
            item = self.flCtrl.GetNextItem(item,
                                           wx.LIST_NEXT_ALL,
                                           wx.LIST_STATE_SELECTED)
            indexes.append(item)
            if item == -1:
                indexes.remove(-1)
                break

        if self.flCtrl.GetItemCount() == len(indexes):
            self.delete_all(self)
            return

        for num in sorted(indexes, reverse=True):
            self.flCtrl.DeleteItem(num)  # remove selected items
            self.data.pop(num)  # remove selected items
            self.outputnames.pop(num)  # remove selected items
            self.file_src.pop(num)
            self.duration.pop(num)
            self.flCtrl.Select(num - 1)  # select the previous one
        self.changes_in_progress(setfocus=False)  # reset timeline
        # self.on_deselect(self)  # deselect removed file

        for x in range(self.flCtrl.GetItemCount()):
            self.flCtrl.SetItem(x, 0, str(x + 1))  # re-load counter
        return
    # ----------------------------------------------------------------------

    def delete_all(self, event, setstate=True):
        """
        Clear all lines on the listCtrl and delete
        self.data list. If already empty, return None.
        """
        if self.flCtrl.GetItemCount() == 0:
            return
        # self.flCtrl.ClearAll()
        self.flCtrl.DeleteAllItems()
        del self.data[:]
        del self.outputnames[:]
        del self.file_src[:]
        del self.duration[:]
        if event:
            self.changes_in_progress(setfocus=False)
            self.parent.rename.Enable(False)
            self.parent.rename_batch.Enable(False)
            self.parent.filedropselected = None
            pub.sendMessage("RESET_ON_CHANGED_LIST", msg=None)
        if setstate:
            self.sortingstate = None
            self.parent.toolbar.EnableTool(9, False)
    # ----------------------------------------------------------------------

    def on_select(self, event):
        """
        Selecting line with mouse or up/down keyboard buttons
        """
        index = self.flCtrl.GetFocusedItem()
        item = self.flCtrl.GetItemText(index, 1)
        self.parent.filedropselected = item
        self.parent.rename.Enable(True)
        pub.sendMessage("RESET_ON_CHANGED_LIST", msg=index)
    # ----------------------------------------------------------------------

    def on_deselect(self, event):
        """
        Event to deselect a line when clicking
        in an empty space of the control list
        """
        self.parent.filedropselected = None
        self.parent.rename.Enable(False)
        pub.sendMessage("RESET_ON_CHANGED_LIST", msg=None)
    # ----------------------------------------------------------------------

    def on_file_save(self, path):
        """
        Set a specific directory for files saving
        """
        self.text_path_save.SetValue("")
        self.text_path_save.AppendText(path)
    # -----------------------------------------------------------------------

    def renaming_file(self):
        """
        This method is responsible for renaming the selected
        output file.
        Names consisting of only whitespaces or tabs or matching
        same name as outputnames are rejected silently.
        """
        row_id = self.flCtrl.GetFocusedItem()  # Get the current row
        oldname = self.flCtrl.GetItemText(row_id, 5)  # Get current name
        newname = ''
        title = _('File renaming...')
        msg = _('Rename the selected file to:')

        with Renamer(self,
                     nameprop=oldname,
                     caption=title,
                     message=msg,
                     mode=0,
                     ) as dlg:
            if dlg.ShowModal() == wx.ID_OK:
                newname = dlg.getvalue()
            else:
                return

        if newname == '' or newname == oldname or newname.isspace():
            self.parent.statusbar_msg(_('Add Files'), None)
            return

        sanitize = filename_sanitize(newname, self.outputnames)
        if sanitize:
            self.parent.statusbar_msg(sanitize, FileDnD.YELLOW, FileDnD.BLACK)
            return

        self.flCtrl.SetItem(row_id, 5, newname)
        self.outputnames[row_id] = newname
        self.parent.statusbar_msg(_('Add Files'), None)
# -----------------------------------------------------------------------

    def renaming_batch_files(self):
        """
        This method is responsible for batch file renaming.
        """
        title = _('Rename items')
        msg = _('Rename the {0} items to:').format(len(self.outputnames))
        with Renamer(self,
                     nameprop=_('New Name #'),
                     caption=title,
                     message=msg,
                     mode=len(self.outputnames)
                     ) as dlg:
            if dlg.ShowModal() == wx.ID_OK:
                newname = dlg.getvalue()
            else:
                return

        for name in newname:
            sanitize = filename_sanitize(name, self.outputnames)
            if sanitize:
                self.parent.statusbar_msg(sanitize, FileDnD.YELLOW,
                                          FileDnD.BLACK)
                return

        for num, name in enumerate(newname):
            self.flCtrl.SetItem(num, 5, name)
            self.outputnames[num] = name

        self.parent.statusbar_msg(_('Add Files'), None)



# -*- coding: UTF-8 -*-
"""
Name: msg_info.py
Porpose: Gets information about the program
Compatibility: Python3, Python2
author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
lic: GPL3
Rev: Mar.01.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""


def current_release():
    """
    General info strings
    NOTE: number version > major number.minor number.micro
    number(patch number) the sub release a=alpha release, b=beta
    release, c= candidate release
    Example 19.1.1c1
    """
    release_name = 'Videomass'
    program_name = 'videomass'
    version = '5.0.11'
    release = 'released'
    copyr = '2013-2024'
    website = 'http://jeanslack.github.io/Videomass/'
    author = ('Gianluca Pernigotto', '(aka jeanslack)')
    mail = 'jeanlucperni@gmail.com'
    comment = ("\nThanks to:\n"
               "- Python <https://www.python.org/>, programming language\n"
               "- wxPython <https://wxpython.org/>, cross-platform\n"
               "GUI toolkit for the Python language\n"
               "- FFmpeg, FFmpeg is a trademark of Fabrice Bellard, \n"
               "originator of the FFmpeg project:\n"
               "<http://ffmpeg.org/>\n"
               "- yt-dlp: <https://github.com/yt-dlp/yt-dlp>\n"
               "Download videos from YouTube and more sites\n"
               )
    return (release_name, program_name, version, release,
            copyr, website, author, mail, comment)


def descriptions_release():
    """
    General info string
    """
    copyr = current_release()
    author = current_release()
    mail = current_release()

    short_d = "Videomass is a cross-platform GUI for FFmpeg and yt-dlp"

    long_d = """
Videomass is a powerful, multitasking and cross-platform graphical user
interface (GUI) for FFmpeg and yt-dlp. It offers a wide range of features and
functions, making it a comprehensive software solution.

Videomass is written in Python3 using the wxPython4-Phoenix toolkit.
"""

    short_l = "GPL3 (Gnu Public License)"

    lic = (f"Copyleft - all rights reversed - {copyr[4]} {author[6][0]}\n"
           f"Author and developer: {author[6][0]} {author[6][1]}\n"
           f"Mail: {mail[7]}\n\n"
           "Videomass is free software: you can redistribute\n"
           "it and/or modify it under the terms of the GNU General\n"
           "Public License as published by the Free Software\n"
           "Foundation, either version 3 of the License, or (at your\n"
           "option) any later version.\n\n"

           "Videomass is distributed in the hope that it\n"
           "will be useful, but WITHOUT ANY WARRANTY; without\n"
           "even the implied warranty of MERCHANTABILITY or\n"
           "FITNESS FOR A PARTICULAR PURPOSE.\n"
           "See the GNU General Public License for more details.\n\n"

           "You should have received a copy of the GNU General\n"
           "Public License along with this program. If not, see\n"
           "http://www.gnu.org/licenses/"
           )
    return (short_d, long_d, short_l, lic)



# -*- coding: UTF-8 -*-
"""
Name: long_processing_task.py
Porpose: Console to show logging messages during processing
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.22.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import time
import os
from shutil import move
from pubsub import pub
import wx
from videomass.vdms_dialogs.widget_utils import notification_area
from videomass.vdms_io.make_filelog import make_log_template
from videomass.vdms_threads.ffmpeg import FFmpeg
from videomass.vdms_threads.picture_exporting import PicturesFromVideo
from videomass.vdms_threads.concat_demuxer import ConcatDemuxer
from videomass.vdms_threads.slideshow import SlideshowMaker
from videomass.vdms_utils.utils import (time_to_integer, integer_to_time)
from videomass.vdms_io import io_tools


def delete_file_source(flist, trashdir):
    """
    Move whole files list to Videomass Trash folder
    after encoding process.
    """
    filenotfounderror = None
    if not os.path.exists(trashdir):
        if not os.path.isdir(trashdir):
            try:
                os.mkdir(trashdir, mode=0o777)
            except FileNotFoundError as err:
                filenotfounderror = err

    if filenotfounderror is not None:
        wx.MessageBox(f"{filenotfounderror}", 'Videomass', wx.ICON_ERROR)
    else:
        date = time.strftime('%H%M%S-%a_%d_%B_%Y')
        for name in flist:
            dest = os.path.join(trashdir, f'{date}_{os.path.basename(name)}')
            move(name, dest)


def pairwise(iterable):
    """
    Return a zip object from iterable.
    This function is used by the update_display method.
    ----
    USE:

    after splitting ffmpeg's progress strings such as:
    output = "frame= 1178 fps=155 q=29.0 size=    2072kB time=00:00:39.02
              bitrate= 435.0kbits/s speed=5.15x  "
    in a list as:
    iterable = ['frame', '1178', 'fps', '155', 'q', '29.0', 'size', '2072kB',
                'time', '00:00:39.02', 'bitrate', '435.0kbits/s', speed',
                '5.15x']
    1)
        for x, y in pairwise(iterable):
            x, y
    2)
        dict(pairwise(iterable))

    Return: a zip object pairs from list iterable object.

    <https://stackoverflow.com/questions/5389507/iterating-over-every-
    two-elements-in-a-list>
    """
    itobj = iter(iterable)  # list_iterator object
    return zip(itobj, itobj)  # zip object pairs from list iterable object
# ----------------------------------------------------------------------#


class LogOut(wx.Panel):
    """
    displays a text control for the output logging, a progress bar
    and a progressive percentage text label. This panel is used
    in combination with separated threads for long processing tasks.
    It also implements stop and close buttons to stop the current
    process and close the panel at the end.

    """
    # used msg on text
    MSG_done = _('[Videomass]: SUCCESS !')
    MSG_failed = _('[Videomass]: FAILED !')
    MSG_taskfailed = _('Sorry, all task failed !')
    MSG_fatalerror = _("The process was stopped due to a fatal error.")
    MSG_interrupted = _('Interrupted Process !')
    MSG_completed = _('Successfully completed !')
    MSG_unfinished = _('Not everything was successful.')

    WHITE = '#fbf4f4'  # white for background status bar
    BLACK = '#060505'  # black for background status bar
    YELLOW = '#bd9f00'
    # ------------------------------------------------------------------#

    def __init__(self, parent):
        """
        In the 'previous' attribute is stored an ID string used to
        recover the previous panel from which the process is started.
        The 'logname' attribute is the name_of_panel.log file in which
        log messages will be written

        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.parent = parent  # main frame
        self.thread_type = None  # the instantiated thread
        self.with_eta = True  # create estimated time of arrival (ETA)
        self.abort = False  # if True set to abort current process
        self.error = False  # if True, all the tasks was failed
        self.previous = None  # panel name from which it starts
        self.logfile = None  # log pathname, None otherwise
        self.result = []  # result of the final process
        self.count = 0  # keeps track of the counts (see `update_count`)
        self.maxrotate = 0  # max num text rotation (see `update_count`)
        self.clr = self.appdata['icontheme'][1]

        wx.Panel.__init__(self, parent=parent)

        infolbl = _("Encoding Status")
        lbl = wx.StaticText(self, label=infolbl)
        if self.appdata['ostype'] != 'Darwin':
            lbl.SetLabelMarkup(f"<b>{infolbl}</b>")
        self.btn_viewlog = wx.Button(self, wx.ID_ANY, _("Current Log"),
                                     size=(-1, -1))
        self.btn_viewlog.Disable()
        self.txtout = wx.TextCtrl(self, wx.ID_ANY, "",
                                  style=wx.TE_MULTILINE
                                  | wx.TE_READONLY
                                  | wx.TE_RICH2
                                  )
        self.barprog = wx.Gauge(self, wx.ID_ANY, range=0)
        self.labprog = wx.StaticText(self, label="")
        self.labffmpeg = wx.StaticText(self, label="")
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0, 10))
        sizer.Add(lbl, 0, wx.ALL, 5)
        sizer.Add(self.txtout, 1, wx.EXPAND | wx.ALL, 5)
        sizer.Add(self.btn_viewlog, 0, wx.ALL, 5)
        sizer.Add(self.barprog, 0, wx.EXPAND | wx.ALL, 5)
        sizer.Add(self.labprog, 0, wx.ALL, 5)
        sizer.Add(self.labffmpeg, 0, wx.ALL, 5)
        line = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                             size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                             name=wx.StaticLineNameStr,
                             )
        sizer.Add(line, 0, wx.ALL | wx.EXPAND, 5)
        # set_properties:
        self.txtout.SetBackgroundColour(self.clr['BACKGRD'])
        self.SetSizerAndFit(sizer)
        # ------------------------------------------
        self.Bind(wx.EVT_BUTTON, self.view_log, self.btn_viewlog)

        pub.subscribe(self.update_display, "UPDATE_EVT")
        pub.subscribe(self.update_count, "COUNT_EVT")
        pub.subscribe(self.end_proc, "END_EVT")
    # ----------------------------------------------------------------------

    def view_log(self, event):
        """
        Opens the log file corresponding to the last executed process.
        """
        if self.logfile:
            fname = str(self.logfile)
            if os.path.exists(fname) and os.path.isfile(fname):
                io_tools.openpath(fname)
    # ----------------------------------------------------------------------

    def topic_thread(self, args, data, previous='View', mode='w'):
        """
        This method is resposible to create the Thread instance.

        """
        self.previous = previous  # stores the panel from which it starts

        if args[0] == 'View':
            return

        self.txtout.Clear()
        self.labprog.SetLabel('')
        self.labffmpeg.SetLabel('')
        self.btn_viewlog.Disable()

        self.logfile = make_log_template(args[1],
                                         self.appdata['logdir'],
                                         mode,  # w or a
                                         )
        if args[0] in ('One pass', 'Two pass', 'Two pass EBU',
                       'Two pass VIDSTAB', 'Queue Processing'):
            self.thread_type = FFmpeg(self.logfile, data)

        elif args[0] == 'video_to_sequence':
            self.with_eta, self.maxrotate = False, None
            self.thread_type = PicturesFromVideo(self.logfile, **data)

        elif args[0] == 'sequence_to_video':
            self.maxrotate = None
            self.thread_type = SlideshowMaker(self.logfile, **data)

        elif args[0] == 'concat_demuxer':
            self.with_eta, self.maxrotate = False, None
            self.thread_type = ConcatDemuxer(self.logfile, **data)
    # ----------------------------------------------------------------------

    def update_display(self, output, duration, status):
        """
        Receive message from thread by pubsub UPDATE_EVT protocol.
        The received 'output' is parsed for calculate the bar
        progress value, percentage label and errors management.
        This method can be used even for non-loop threads.

        NOTE: During conversion the ffmpeg errors do not stop all
              others tasks, if an error occurred it will be marked
              with 'failed' but the other tasks will continue;
              if it has finished without errors it will be marked with
              'done' on `update_count` method. Since not all ffmpeg
              messages are errors, sometimes it happens to see more
              output marked with yellow color.
        """
        if not status == 0:  # error, exit status of the p.wait
            self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['ERR1']))
            self.txtout.AppendText(f"{LogOut.MSG_failed}\n")
            self.result.append('failed')
            return  # must be return here

        if 'time=' in output:  # ...in processing
            i = output.index('time=') + 5
            pos = output[i:].split()[0]
            msec = time_to_integer(pos)

            if msec > duration:
                self.barprog.SetValue(duration)
            elif msec == 0:
                self.barprog.SetValue(self.barprog.GetValue())
            else:
                self.barprog.SetValue(msec)

            percentage = round((msec / duration) * 100 if
                               duration != 0 else 100)
            out = [a for a in "=".join(output.split()).split('=') if a]
            ffprog = []
            for key, val in pairwise(out):
                ffprog.append(f"{key}: {val}")

            if self.with_eta:
                if 'speed=' in output:
                    speed = output.split('speed=')[-1].strip().split('x')[0]
                    if speed in ('N/A', '0'):
                        eta = "   ETA: N/A"
                    else:  # is float
                        rem = (duration - msec) / float(speed)
                        remaining = integer_to_time(round(rem))
                        eta = f"   ETA: {remaining}"
                else:
                    eta = "   ETA: N/A"
            else:
                eta = ""
            self.labprog.SetLabel(f'Processing: {str(int(percentage))}% {eta}')
            self.labffmpeg.SetLabel(' | '.join(ffprog))

            del output, duration

        else:  # append all others lines on the textctrl and log file
            if [x for x in ('info', 'Info') if x in output]:
                self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['INFO']))
                self.txtout.AppendText(f'{output}')

            elif [x for x in ('Failed', 'failed', 'Error', 'error')
                    if x in output]:
                self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['ERR0']))
                self.txtout.AppendText(f'{output}')

            elif [x for x in ('warning', 'Warning', 'warn') if x in output]:
                self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['WARN']))
                self.txtout.AppendText(f'{output}')

            else:
                self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['TXT3']))
                self.txtout.AppendText(f'{output}')

            with open(self.logfile, "a", encoding='utf8') as logerr:
                logerr.write(f"[FFMPEG]: {output}")
    # ----------------------------------------------------------------------

    def update_count(self, count, duration, end):
        """
        Receive messages from file count, loop or non-loop thread.
        """
        if end == 'Done':
            self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['SUCCESS']))
            self.txtout.AppendText(f"{LogOut.MSG_done}\n")
            # set end values for percentage and ETA
            if self.with_eta:
                newlab = self.labprog.GetLabel().split()
                if 'Processing:' in newlab:
                    newlab[1] = '100%   '
                if 'ETA:' in newlab:
                    newlab[3] = '00:00:00.000'
                self.labprog.SetLabel(" ".join(newlab))
            else:
                self.labprog.SetLabel('Processing: 100%')
            return

        # if STATUS_ERROR == 1:
        if end == 'error':
            self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['WARN']))
            self.txtout.AppendText(f'\n{count}\n\n')
            self.error = True
        else:
            if self.maxrotate is not None:
                if self.maxrotate == 1:
                    self.maxrotate = 0
                    self.txtout.Clear()
                self.maxrotate += 1
            self.barprog.SetRange(duration)  # set overall duration range
            self.barprog.SetValue(0)  # reset bar progress
            self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['TXT0']))
            self.txtout.AppendText(f'\n{count}\n\n')
        self.count += 1
    # ----------------------------------------------------------------------

    def end_proc(self, filetotrash):
        """
        At the end of the process
        """
        if self.error:
            self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['TXT0']))
            self.txtout.AppendText(f"\n{LogOut.MSG_fatalerror}\n")
            notification_area(_("Fatal Error !"), LogOut.MSG_fatalerror,
                              wx.ICON_ERROR)
        elif self.abort:
            self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['ABORT']))
            self.txtout.AppendText(f"\n{LogOut.MSG_interrupted}\n")
        else:
            if not self.result:
                endmsg = LogOut.MSG_completed
                self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['TXT0']))
                notification_area(endmsg, _("Get your files at the "
                                            "destination you specified"),
                                  wx.ICON_INFORMATION,
                                  )
                if os.path.basename(self.logfile) == 'Queue Processing.log':
                    pub.sendMessage("QUEUE PROCESS SUCCESSFULLY", msg='Done')

            else:
                if len(self.result) == self.count:
                    endmsg = LogOut.MSG_taskfailed
                else:
                    endmsg = LogOut.MSG_unfinished

                self.txtout.SetDefaultStyle(wx.TextAttr(self.clr['TXT0']))
                notification_area(endmsg, _("For more details please read the "
                                            "Current Log."), wx.ICON_ERROR)

            self.parent.statusbar_msg(_('...Finished'), None)
            self.txtout.AppendText(f"\n{endmsg}\n")
            self.barprog.SetValue(0)

            if filetotrash:  # move processed files to Videomass trash folder
                if self.parent.movetotrash:
                    trashdir = self.appdata['user_trashdir']
                    delete_file_source(filetotrash, trashdir)  # filelist, dir

        self.txtout.AppendText('\n')
        self.reset_all()
        pub.sendMessage("PROCESS TERMINATED", msg='Terminated')
    # ----------------------------------------------------------------------

    def on_stop(self):
        """
        The user change idea and was stop process
        """
        self.thread_type.stop()
        self.parent.statusbar_msg(_("Please wait... interruption in progress"),
                                  LogOut.YELLOW, LogOut.BLACK)
        self.thread_type.join()
        self.parent.statusbar_msg(_("...Interrupted"), None)
        self.abort = True
        # event.Skip()
    # ----------------------------------------------------------------------

    def reset_all(self):
        """
        Reset to default at any process terminated
        """
        self.thread_type = None
        self.abort = False
        self.error = False
        self.result.clear()
        self.count = 0
        self.maxrotate = 0
        self.with_eta = True  # restoring time remaining display
        self.btn_viewlog.Enable()
    # ----------------------------------------------------------------------



# -*- coding: UTF-8 -*-
"""
Name: textdrop.py
Porpose: Allows you to add text URLs
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.23.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from urllib.parse import urlparse
import wx
from videomass.vdms_dialogs.list_warning import ListWarning


class MyListCtrl(wx.ListCtrl):
    """
    This is the listControl widget.
    Note that this wideget has DnDPanel parented.
    """
    def __init__(self, parent):
        """
        Constructor.
        WARNING to avoid segmentation error on removing items by
        listctrl, style must be wx.LC_SINGLE_SEL .
        """
        self.index = None
        self.parent = parent  # parent is DnDPanel class
        self.errors = {}
        wx.ListCtrl.__init__(self,
                             parent,
                             style=wx.LC_REPORT
                             | wx.LC_SINGLE_SEL,
                             )
        self.populate()
    # ----------------------------------------------------------------------#

    def populate(self):
        """
        make default colums
        """
        self.InsertColumn(0, ('#'), width=30)
        self.InsertColumn(1, (_('Url')), width=600)

    def dropUpdate(self, url):
        """
        Update list-control during drag and drop.
        """
        self.index = self.GetItemCount()
        res = urlparse(url)
        if not res[1]:  # if empty netloc given from ParseResult
            self.errors[f'{url}'] = _('Invalid URL')
            return False

        self.InsertItem(self.index, str(self.index + 1))
        self.SetItem(self.index, 1, url)
        self.index += 1

        self.parent.changes_in_progress()
        return True
    # ----------------------------------------------------------------------#

    def rejected_urls(self):
        """
        Handles all rejected URLs if any
        """
        if self.errors:
            msg = _('Detailed list of errors')
            with ListWarning(self,
                             self.errors,
                             caption=_('Invalid URLs'),
                             header=msg,
                             buttons='OK',
                             ) as log:
                log.ShowModal()
            self.errors.clear()  # clear values here


class UrlDropTarget(wx.TextDropTarget):
    """
    This is a Drop target object for handle dragging
    text URL data on a ListCtrl object.
    """
    def __init__(self, parent, listctrl):
        """
        Defining the ListCtrl object attribute
        """
        wx.TextDropTarget.__init__(self)
        self.parent = parent
        self.listctrl = listctrl

    def OnDropText(self, x, y, data):
        """
        Update ListCtrl object by dragging text inside it.
        """
        for url in data.split():
            self.listctrl.dropUpdate(url)
        self.listctrl.rejected_urls()

        return True


class Url_DnD_Panel(wx.Panel):
    """
    Panel responsible to embed URLs controls
    """
    def __init__(self, parent):
        """
        parent is the MainFrame
        """
        self.parent = parent
        get = wx.GetApp()
        # colors = get.appset['icontheme'][1]

        wx.Panel.__init__(self, parent, -1)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0, 25))
        infomsg = _("Drag or paste URLs here")
        lbl_info = wx.StaticText(self, wx.ID_ANY, label=infomsg)
        sizer.Add(lbl_info, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add((0, 10))
        self.urlctrl = MyListCtrl(self)
        dragt = UrlDropTarget(self, self.urlctrl)
        self.urlctrl.SetDropTarget(dragt)
        sizer.Add(self.urlctrl, 1, wx.EXPAND | wx.ALL, 5)

        sizer_ctrl = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(sizer_ctrl, 0, wx.ALL | wx.EXPAND, 0)
        self.text_path_save = wx.TextCtrl(self, wx.ID_ANY, "",
                                          style=wx.TE_PROCESS_ENTER
                                          | wx.TE_READONLY,
                                          )
        sizer_ctrl.Add(self.text_path_save, 1, wx.LEFT | wx.EXPAND, 5)

        self.btn_save = wx.Button(self, wx.ID_OPEN, "...", size=(35, -1))
        sizer_ctrl.Add(self.btn_save, 0, wx.RIGHT | wx.LEFT
                       | wx.ALIGN_CENTER_HORIZONTAL
                       | wx.ALIGN_CENTER_VERTICAL, 5,
                       )
        self.SetSizer(sizer)

        if get.appset['ostype'] == 'Darwin':
            lbl_info.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        else:
            lbl_info.SetFont(wx.Font(9, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.text_path_save.SetValue(self.parent.outputdir)
        # Tooltip
        self.btn_save.SetToolTip(_("Set a new destination for your downloads"))
        self.text_path_save.SetToolTip(_("Current destination directory"))
        self.Bind(wx.EVT_CONTEXT_MENU, self.onContext)
    # ---------------------------------------------------------

    def onContext(self, event):
        """
        Create and show a Context Menu
        """
        # only do this part the first time so the events are only bound once
        if not hasattr(self, "popupID1"):
            popupID1 = wx.ID_ANY
            popupID2 = wx.ID_ANY
            popupID3 = wx.ID_ANY
            self.Bind(wx.EVT_MENU, self.onPopup, id=popupID1)
            self.Bind(wx.EVT_MENU, self.onPopup, id=popupID2)
            self.Bind(wx.EVT_MENU, self.onPopup, id=popupID3)
        # build the menu
        menu = wx.Menu()
        menu.Append(popupID2, _("Paste\tCtrl+V"))
        menu.Append(popupID1, _("Remove selected URL\tDEL"))
        menu.Append(popupID3, _("Clear list\tShift+DEL"))
        # show the popup menu
        self.PopupMenu(menu)
        menu.Destroy()
    # ----------------------------------------------------------------------

    def onPopup(self, event):
        """
        Evaluate the label string of the menu item selected and starts
        the related process
        """
        itemId = event.GetId()
        menu = event.GetEventObject()
        menuItem = menu.FindItemById(itemId)

        if menuItem.GetItemLabel() == _("Paste\tCtrl+V"):
            self.on_paste(self)

        elif menuItem.GetItemLabel() == _("Remove selected URL\tDEL"):
            self.on_del_url_selected(self)

        elif menuItem.GetItemLabel() == _("Clear list\tShift+DEL"):
            self.delete_all(self)
    # ----------------------------------------------------------------------

    def changes_in_progress(self, setfocus=True):
        """
        Update new URLs list by drag&drop operations.
        """
        if setfocus:
            sel = self.urlctrl.GetFocusedItem()  # Get the current row
            selitem = sel if sel != -1 else 0
            self.urlctrl.Focus(selitem)  # make the line the current line
            self.urlctrl.Select(selitem, on=1)  # default event selection

        data = []
        for x in range(self.urlctrl.GetItemCount()):
            data.append(self.urlctrl.GetItemText(x, 1))

        if not data == self.parent.data_url:
            self.parent.changed = True

        self.parent.statusbar_msg(_('Ready'), None)
        self.parent.data_url = data.copy()
        self.parent.destroy_orphaned_window()
        self.parent.toolbar.EnableTool(25, True)
    # -----------------------------------------------------------------------

    def on_paste(self, event):
        """
        Event on clicking paste button to paste
        text data on the ListCtrl
        """
        text_data = wx.TextDataObject()
        if wx.TheClipboard.Open():
            success = wx.TheClipboard.GetData(text_data)
            wx.TheClipboard.Close()
        if success:
            data = text_data.GetText().split()
            for url in data:
                self.urlctrl.dropUpdate(url)
            self.urlctrl.rejected_urls()
    # ----------------------------------------------------------------------

    def delete_all(self, event):
        """
        Clear all text lines of the TxtCtrl.
        If already empty, return None
        """
        if self.urlctrl.GetItemCount() == 0:
            return
        self.urlctrl.DeleteAllItems()
        self.parent.destroy_orphaned_window()
        self.parent.toolbar.EnableTool(25, False)
        del self.parent.data_url[:]
    # -----------------------------------------------------------

    def on_del_url_selected(self, event):
        """
        Delete a selected url, if nothing is selected return None
        """
        if self.urlctrl.GetFirstSelected() == -1:  # None
            return

        item, indexes = -1, []
        while 1:
            item = self.urlctrl.GetNextItem(item,
                                            wx.LIST_NEXT_ALL,
                                            wx.LIST_STATE_SELECTED)
            indexes.append(item)
            if item == -1:
                indexes.remove(-1)
                break

        if self.urlctrl.GetItemCount() == len(indexes):
            self.delete_all(self)
            return

        for num in sorted(indexes, reverse=True):
            self.urlctrl.DeleteItem(num)  # remove selected items
            self.urlctrl.Select(num - 1)  # select the previous one
        self.changes_in_progress(setfocus=False)

        for x in range(self.urlctrl.GetItemCount()):
            self.urlctrl.SetItem(x, 0, str(x + 1))  # re-load counter
        return
    # ----------------------------------------------------------------------

    def on_file_save(self, path):
        """
        Set a specific directory for files saving
        """
        self.text_path_save.SetValue("")
        self.text_path_save.AppendText(path)
        self.parent.outputdir = path



# -*- coding: UTF-8 -*-
"""
Name: wizard_dlg.py
Porpose: wizard setup dialog
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: April.18.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import wx
from videomass.vdms_utils.utils import detect_binaries
from videomass.vdms_sys.settings_manager import ConfigManager


def write_changes(fileconf, ffmpeg, ffplay, ffprobe, youtubedl, binfound):
    """
    Writes changes to the configuration file

    """
    conf = ConfigManager(fileconf)
    dataread = conf.read_options()
    dataread['ffmpeg_cmd'] = ffmpeg
    dataread['ffprobe_cmd'] = ffprobe
    dataread['ffplay_cmd'] = ffplay
    # local = False if binfound == 'system' else True
    local = not binfound == 'system'
    dataread['ffmpeg_islocal'] = local
    dataread['ffprobe_islocal'] = local
    dataread['ffplay_islocal'] = local
    dataread['use-downloader'] = youtubedl
    conf.write_options(**dataread)


class PageOne(wx.Panel):
    """
    This is a first panel displayed on Wizard dialog box

    """
    get = wx.GetApp()
    OS = get.appset['ostype']
    MSG2 = _("Please take a moment to set up the application")
    MSG3 = _('Click the "Next" button to get started')

    def __init__(self, parent, icon):
        """
        constructor
        """
        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)

        sizer_base = wx.BoxSizer(wx.VERTICAL)
        bitmap = wx.Bitmap(icon, wx.BITMAP_TYPE_ANY)
        img = bitmap.ConvertToImage()
        img = img.Scale(64, 64, wx.IMAGE_QUALITY_NORMAL)
        img = img.ConvertToBitmap()
        bitmap_vdms = wx.StaticBitmap(self, wx.ID_ANY, img)
        lab1 = wx.StaticText(self, wx.ID_ANY,
                             _("Welcome to the Videomass Wizard!"))
        lab2 = wx.StaticText(self, wx.ID_ANY, PageOne.MSG2,
                             style=wx.ST_ELLIPSIZE_END
                             | wx.ALIGN_CENTRE_HORIZONTAL,
                             )
        lab3 = wx.StaticText(self, wx.ID_ANY, PageOne.MSG3,
                             style=wx.ALIGN_CENTRE_HORIZONTAL)

        if PageOne.OS == 'Darwin':
            lab1.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
            lab2.SetFont(wx.Font(13, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
        else:
            lab1.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
            lab2.SetFont(wx.Font(11, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))

        # layout
        sizer_base.Add((0, 80), 0)
        sizer_base.Add(bitmap_vdms, 0, wx.CENTER)
        sizer_base.Add((0, 30), 0)
        sizer_base.Add(lab1, 0, wx.CENTER)
        sizer_base.Add((0, 30), 0)
        sizer_base.Add(lab2, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 30), 0)
        sizer_base.Add(lab3, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 80), 0)

        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()


class PageTwo(wx.Panel):
    """
    Shows panel wizard to locate FFmpeg executables
    and set the Wizard attributes on parent.

    """
    get = wx.GetApp()
    OS = get.appset['ostype']
    GETPATH = get.appset['getpath']
    FFMPEG_LOCALDIR = get.appset['FFMPEG_videomass_pkg']

    MSG0 = _('Videomass is an application based on FFmpeg\n')

    MSG1 = (_('If FFmpeg is not on your computer, this application '
              'will be unusable'))

    MSG2 = (_('If you have already installed FFmpeg on your operating\n'
              'system, click the "Auto-detection" button.'))

    MSG3 = (_('If you want to use a version of FFmpeg located on your\n'
              'filesystem but not installed on your operating system,\n'
              'click the "Locate" button.'))

    def __init__(self, parent):
        """
        constructor
        """
        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)

        self.parent = parent

        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sizerText = wx.BoxSizer(wx.VERTICAL)
        lab0 = wx.StaticText(self, wx.ID_ANY, PageTwo.MSG0,
                             style=wx.ST_ELLIPSIZE_END
                             | wx.ALIGN_CENTRE_HORIZONTAL,
                             )
        lab1 = wx.StaticText(self, wx.ID_ANY, PageTwo.MSG1,
                             style=wx.ALIGN_CENTRE_HORIZONTAL)
        lab2 = wx.StaticText(self, wx.ID_ANY, PageTwo.MSG2)
        lab3 = wx.StaticText(self, wx.ID_ANY, PageTwo.MSG3)
        self.detectBtn = wx.Button(self, wx.ID_ANY, _("Auto-detection"),
                                   size=(250, -1))
        self.locateBtn = wx.Button(self, wx.ID_ANY, _("Locate"),
                                   size=(250, -1))
        self.labFFpath = wx.StaticText(self, wx.ID_ANY, "",
                                       style=wx.ST_ELLIPSIZE_END
                                       | wx.ALIGN_CENTRE_HORIZONTAL,
                                       )
        if PageTwo.OS == 'Darwin':
            lab0.SetFont(wx.Font(14, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
            lab1.SetFont(wx.Font(10, wx.SWISS, wx.ITALIC, wx.NORMAL, 0, ""))
            self.labFFpath.SetFont(wx.Font(13, wx.MODERN,
                                           wx.NORMAL, wx.BOLD, 0, ""))
        else:
            lab0.SetFont(wx.Font(12, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
            lab1.SetFont(wx.Font(8, wx.SWISS, wx.ITALIC, wx.NORMAL, 0, ""))
            self.labFFpath.SetFont(wx.Font(10, wx.MODERN,
                                           wx.NORMAL, wx.BOLD, 0, ""))
        # layout
        sizer_base.Add((0, 50), 0)
        sizer_base.Add(lab0, 0, wx.CENTER | wx.EXPAND)
        # sizer_base.Add((0, 5), 0)
        sizer_base.Add(lab1, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 40), 0)
        sizer_base.Add(sizerText, 0, wx.CENTER)
        sizerText.Add(lab2, 0, wx.EXPAND)
        sizerText.Add((0, 15), 0)
        sizerText.Add(lab3, 0, wx.EXPAND)
        sizer_base.Add((0, 40), 0)
        sizer_base.Add(self.detectBtn, 0, wx.CENTER)
        sizer_base.Add((0, 5), 0)
        sizer_base.Add(self.locateBtn, 0, wx.CENTER)
        sizer_base.Add((0, 25), 0)
        sizer_base.Add(self.labFFpath, 0, wx.CENTER | wx.EXPAND)

        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()

        # bindings
        self.Bind(wx.EVT_BUTTON, self.detectbin, self.detectBtn)
        self.Bind(wx.EVT_BUTTON, self.Locate, self.locateBtn)
    # -------------------------------------------------------------------#

    def Locate(self, event):
        """
        The user browse manually to find ffmpeg, ffprobe,
        ffplay executables

        """
        self.parent.btnNext.Enable()
        self.locateBtn.Disable()
        self.detectBtn.Enable()
        self.labFFpath.SetLabel(_('Click the "Next" button'))
        self.Layout()
    # -------------------------------------------------------------------#

    def detectbin(self, event):
        """
        The user push the auto-detect button to automatically
        detect ffmpeg, ffprobe and ffplay on the O.S.

        """
        if PageTwo.OS == 'Windows':
            executable = ['ffmpeg.exe', 'ffprobe.exe', 'ffplay.exe']
        else:
            executable = ['ffmpeg', 'ffprobe', 'ffplay']

        exiting = None
        path = []
        for exe in executable:
            status = detect_binaries(exe, PageTwo.FFMPEG_LOCALDIR)

            if status[0] == 'not installed':
                wx.MessageBox(_("'{}' is not installed on your computer. "
                                "Install it or indicate another location by "
                                "clicking the 'Locate' button.").format(exe),
                              'Videomass: Warning', wx.ICON_EXCLAMATION, self)
                return

            if status[0] == 'provided':
                exiting = status[0]
                path.append(status[1])

            elif not status[0]:
                path.append(status[1])

        if exiting == 'provided':
            if wx.MessageBox(_("Videomass already seems to include "
                               "FFmpeg.\n\nDo you want to use that?"),
                             _('Videomass: Please Confirm'), wx.ICON_QUESTION
                             | wx.CANCEL | wx.YES_NO, self) != wx.YES:
                return

        self.parent.ffmpeg = PageTwo.GETPATH(path[0])
        self.parent.ffprobe = PageTwo.GETPATH(path[1])
        self.parent.ffplay = PageTwo.GETPATH(path[2])
        self.parent.btnNext.Enable()
        self.detectBtn.Disable()
        self.locateBtn.Enable()
        self.labFFpath.SetLabel(f'...Found: "{PageTwo.GETPATH(path[0])}"')
        self.Layout()


class PageThree(wx.Panel):
    """
    Shows panel to locate manually ffmpeg, ffprobe,
    and ffplay executables and set attributes on parent.

    """
    get = wx.GetApp()
    OS = get.appset['ostype']
    GETPATH = get.appset['getpath']

    MSG0 = _('Locating FFmpeg executables\n')

    MSG1 = (_('"ffmpeg", "ffprobe" and "ffplay" are required. Complete all\n'
              'the text boxes below by clicking on the respective buttons.'))

    def __init__(self, parent):
        """
        constructor
        """
        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)

        self.parent = parent

        if PageTwo.OS == 'Windows':
            self.ffmpeg = 'ffmpeg.exe'
            self.ffprobe = 'ffprobe.exe'
            self.ffplay = 'ffplay.exe'
        else:
            self.ffmpeg = 'ffmpeg'
            self.ffprobe = 'ffprobe'
            self.ffplay = 'ffplay'

        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sizerText = wx.BoxSizer(wx.VERTICAL)
        lab0 = wx.StaticText(self, wx.ID_ANY, PageThree.MSG0,
                             style=wx.ST_ELLIPSIZE_END
                             | wx.ALIGN_CENTRE_HORIZONTAL,
                             )
        lab1 = wx.StaticText(self, wx.ID_ANY, PageThree.MSG1)
        #  ffmpeg
        gridffmpeg = wx.BoxSizer(wx.HORIZONTAL)
        self.ffmpegTxt = wx.TextCtrl(self, wx.ID_ANY, "",
                                     style=wx.TE_READONLY)
        gridffmpeg.Add(self.ffmpegTxt, 1, wx.ALL, 5)

        self.ffmpegBtn = wx.Button(self, wx.ID_ANY, "ffmpeg")
        gridffmpeg.Add(self.ffmpegBtn, 0, wx.RIGHT | wx.CENTER, 5)
        #  ffprobe
        gridffprobe = wx.BoxSizer(wx.HORIZONTAL)
        self.ffprobeTxt = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_READONLY)
        gridffprobe.Add(self.ffprobeTxt, 1, wx.ALL, 5)

        self.ffprobeBtn = wx.Button(self, wx.ID_ANY, "ffprobe")
        gridffprobe.Add(self.ffprobeBtn, 0, wx.RIGHT | wx.CENTER, 5)
        #  ffplay
        gridffplay = wx.BoxSizer(wx.HORIZONTAL)
        self.ffplayTxt = wx.TextCtrl(self, wx.ID_ANY, "",
                                     style=wx.TE_READONLY)
        gridffplay.Add(self.ffplayTxt, 1, wx.ALL, 5)

        self.ffplayBtn = wx.Button(self, wx.ID_ANY, "ffplay")
        gridffplay.Add(self.ffplayBtn, 0, wx.RIGHT | wx.CENTER, 5)

        if PageThree.OS == 'Darwin':
            lab0.SetFont(wx.Font(14, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
        else:
            lab0.SetFont(wx.Font(12, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))

        # layout
        sizer_base.Add((0, 50), 0)
        sizer_base.Add(lab0, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 40), 0)
        sizer_base.Add(sizerText, 0, wx.CENTER)
        sizerText.Add(lab1, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 50), 0)
        sizer_base.Add(gridffmpeg, 0, wx.ALL | wx.EXPAND, 5)
        sizer_base.Add((0, 5), 0)
        sizer_base.Add(gridffprobe, 0, wx.ALL | wx.EXPAND, 5)
        sizer_base.Add((0, 5), 0)
        sizer_base.Add(gridffplay, 0, wx.ALL | wx.EXPAND, 5)

        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()
        # bindings
        self.Bind(wx.EVT_BUTTON, self.on_ffmpeg, self.ffmpegBtn)
        self.Bind(wx.EVT_BUTTON, self.on_ffprobe, self.ffprobeBtn)
        self.Bind(wx.EVT_BUTTON, self.on_ffplay, self.ffplayBtn)

    def on_ffmpeg(self, event):
        """
        Open filedialog to locate ffmpeg executable
        """
        with wx.FileDialog(self, _("{} executable").format(self.ffmpeg),
                           "", "", "ffmpeg binary "
                           f"(*{self.ffmpeg})|*{self.ffmpeg}| "
                           f"All files (*.*)|*.*",
                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as dlgfile:

            if dlgfile.ShowModal() == wx.ID_OK:
                if os.path.basename(dlgfile.GetPath()) == self.ffmpeg:
                    self.ffmpegTxt.Clear()
                    ffmpegpath = PageThree.GETPATH(dlgfile.GetPath())
                    self.ffmpegTxt.write(ffmpegpath)
                    self.parent.ffmpeg = ffmpegpath

    def on_ffprobe(self, event):
        """
        Open filedialog to locate ffprobe executable
        """
        with wx.FileDialog(self, _("{} executable").format(self.ffprobe),
                           "", "", "ffprobe binary "
                           f"(*{self.ffprobe})|*{self.ffprobe}| "
                           f"All files (*.*)|*.*",
                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as dlgfile:

            if dlgfile.ShowModal() == wx.ID_OK:
                if os.path.basename(dlgfile.GetPath()) == self.ffprobe:
                    self.ffprobeTxt.Clear()
                    ffprobepath = PageThree.GETPATH(dlgfile.GetPath())
                    self.ffprobeTxt.write(ffprobepath)
                    self.parent.ffprobe = ffprobepath

    def on_ffplay(self, event):
        """
        Open filedialog to locate ffplay executable
        """
        with wx.FileDialog(self, _("{} executable").format(self.ffplay),
                           "", "", "ffplay binary "
                           f"(*{self.ffplay})|*{self.ffplay}| "
                           f"All files (*.*)|*.*",
                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as dlgfile:

            if dlgfile.ShowModal() == wx.ID_OK:
                if os.path.basename(dlgfile.GetPath()) == self.ffplay:
                    self.ffplayTxt.Clear()
                    ffplaypath = PageThree.GETPATH(dlgfile.GetPath())
                    self.ffplayTxt.write(ffplaypath)
                    self.parent.ffplay = ffplaypath


class PageFour(wx.Panel):
    """
    The PageFour panel asks the user if they want
    to enable youtube-dl.

    """
    get = wx.GetApp()
    OS = get.appset['ostype']

    MSG0 = _('Videomass has a simple graphical interface for yt-dlp\n')
    MSG1 = _('This feature allows you to download video and audio from\n'
             'many sites, including YouTube.com and even Facebook.')

    def __init__(self, parent):
        """
        NOTE: note the pass statement on `choose_Youtubedl`
        the values of this panel are get by Wizard.wizard_Finish method
        """
        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)

        self.parent = parent
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sizerText = wx.BoxSizer(wx.VERTICAL)

        lab0 = wx.StaticText(self, wx.ID_ANY, PageFour.MSG0,
                             style=wx.ST_ELLIPSIZE_END
                             | wx.ALIGN_CENTRE_HORIZONTAL,
                             )
        lab1 = wx.StaticText(self, wx.ID_ANY, PageFour.MSG1)
        descr = _(" Do you want to enable this feature?")
        self.ckbx_yn = wx.CheckBox(self, wx.ID_ANY, (descr))

        if PageFour.OS == 'Darwin':
            lab0.SetFont(wx.Font(14, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
        else:
            lab0.SetFont(wx.Font(12, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
        # layout
        sizer_base.Add((0, 50), 0)
        sizer_base.Add(lab0, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 40), 0)

        sizer_base.Add(sizerText, 0, wx.CENTER)
        sizerText.Add(lab1, 0, wx.CENTER | wx.EXPAND)
        sizerText.Add((0, 50), 0)
        sizerText.Add(self.ckbx_yn, 0, wx.CENTER | wx.ALL, 10)
        sizerText.Add((0, 50), 0)

        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()


class PageFinish(wx.Panel):
    """
    This is last panel to show during wizard session

    """
    get = wx.GetApp()
    OS = get.appset['ostype']
    MSG0 = _("Wizard completed successfully!\n")
    MSG1 = (_("Remember that you can always change these settings "
              "later, through the Setup dialog."))
    MSG3 = _("Thank You!")
    MSG2 = _('To exit click the "Finish" button')

    def __init__(self, parent):
        """
        constructor
        """
        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)

        sizer_base = wx.BoxSizer(wx.VERTICAL)
        lab0 = wx.StaticText(self, wx.ID_ANY, PageFinish.MSG0,
                             style=wx.ALIGN_CENTRE_HORIZONTAL
                             )
        lab1 = wx.StaticText(self, wx.ID_ANY, PageFinish.MSG1,
                             style=wx.ALIGN_CENTRE_HORIZONTAL
                             )
        lab2 = wx.StaticText(self, wx.ID_ANY, PageFinish.MSG2,
                             style=wx.ALIGN_CENTRE_HORIZONTAL
                             )
        lab3 = wx.StaticText(self, wx.ID_ANY, PageFinish.MSG3,
                             style=wx.ALIGN_CENTRE_HORIZONTAL
                             )
        if PageFinish.OS == 'Darwin':
            lab0.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
            lab1.SetFont(wx.Font(10, wx.SWISS, wx.ITALIC, wx.NORMAL, 0, ""))
            lab3.SetFont(wx.Font(14, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))
        else:
            lab0.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
            lab1.SetFont(wx.Font(8, wx.SWISS, wx.ITALIC, wx.NORMAL, 0, ""))
            lab3.SetFont(wx.Font(12, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, ""))

        # layout
        sizer_base.Add((0, 120), 0)
        sizer_base.Add(lab0, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 5), 0)
        sizer_base.Add(lab1, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 30), 0)
        sizer_base.Add(lab3, 0, wx.CENTER | wx.EXPAND)
        sizer_base.Add((0, 30), 0)
        sizer_base.Add(lab2, 0, wx.CENTER | wx.EXPAND)

        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()


class Wizard(wx.Dialog):
    """
    Provides a multi-panel dialog box (dynamic wizard)
    for configuring Videomass during the startup.

    """
    get = wx.GetApp()
    getfileconf = get.appset['fileconfpath']
    OS = get.appset['ostype']

    def __init__(self, icon_videomass):
        """
        Note that the attributes of ffmpeg are set in the "PageTwo"
        and "PageThree" panels. The other values are obtained with
        the `wizard_Finish` method and not on the panels

        """
        self.ffmpeg = None
        self.ffprobe = None
        self.ffplay = None

        wx.Dialog.__init__(self, None, -1,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER,
                           )
        mainSizer = wx.BoxSizer(wx.VERTICAL)  # sizer base global
        self.pageOne = PageOne(self, icon_videomass)  # start...
        self.pageTwo = PageTwo(self)  # choose ffmpeg modality
        self.pageThree = PageThree(self)  # browse for ffmpeg binaries
        self.pageFour = PageFour(self)  # enable or disable youtube-dl
        self.pageFinish = PageFinish(self)  # ...end
        #  hide panels
        self.pageTwo.Hide()
        self.pageThree.Hide()
        self.pageFour.Hide()
        self.pageFinish.Hide()
        #  adds panels to sizer
        mainSizer.Add(self.pageOne, 1, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(self.pageTwo, 1, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(self.pageThree, 1, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(self.pageFour, 1, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(self.pageFinish, 1, wx.ALL | wx.EXPAND, 5)
        # bottom side layout
        gridBtn = wx.GridSizer(1, 2, 0, 0)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        gridBtn.Add(btn_cancel, 0)
        gridchoices = wx.GridSizer(1, 2, 0, 5)
        self.btnBack = wx.Button(self, wx.ID_ANY, _("< Previous"))
        self.btnBack.Disable()
        gridchoices.Add(self.btnBack, 0, wx.EXPAND)
        self.btnNext = wx.Button(self, wx.ID_ANY, _("Next >"))
        gridchoices.Add(self.btnNext, 0, wx.EXPAND)
        gridBtn.Add(gridchoices, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 0)
        mainSizer.Add(gridBtn, 0, wx.ALL | wx.EXPAND, 5)
        #  properties
        self.SetTitle(_("Videomass Wizard"))
        self.SetMinSize((700, 500))
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(icon_videomass, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetSizer(mainSizer)
        self.Fit()
        self.Layout()

        #  bindings
        self.Bind(wx.EVT_BUTTON, self.On_close)
        self.Bind(wx.EVT_BUTTON, self.on_Back, self.btnBack)
        self.Bind(wx.EVT_BUTTON, self.on_Next, self.btnNext)
        self.Bind(wx.EVT_CLOSE, self.On_close)  # controlla la chiusura (x)

    # events:
    def On_close(self, event):
        """
        Destroy app
        """
        self.Destroy()
    # -------------------------------------------------------------------#

    def on_Next(self, event):
        """
        Set the panels to show when the 'Next'
        button is clicked

        """
        if self.btnNext.GetLabel() == _('Finish'):
            self.wizard_Finish()

        if self.pageOne.IsShown():
            self.pageOne.Hide()
            self.pageTwo.Show()
            self.btnBack.Enable()

            if (self.pageTwo.locateBtn.IsEnabled()
                    and self.pageTwo.detectBtn.IsEnabled()):
                self.btnNext.Disable()

        elif self.pageTwo.IsShown():

            if not self.pageTwo.locateBtn.IsEnabled():
                self.pageTwo.Hide()
                self.pageThree.Show()
            else:
                self.pageTwo.Hide()
                self.pageFour.Show()

        elif self.pageThree.IsShown():
            if (self.pageThree.ffmpegTxt.GetValue()
                    and self.pageThree.ffprobeTxt.GetValue()
                    and self.pageThree.ffplayTxt.GetValue()):
                self.pageThree.Hide()
                self.pageFour.Show()
            else:
                wx.MessageBox(_("Some text boxes are still incomplete"),
                              ("Videomass"), wx.ICON_INFORMATION, self)

        elif self.pageFour.IsShown():
            self.pageOne.Hide()
            self.pageTwo.Hide()
            self.pageFour.Hide()
            self.pageFinish.Show()
            self.btnNext.SetLabel(_('Finish'))

        self.Layout()
    # -------------------------------------------------------------------#

    def on_Back(self, event):
        """
        Set the panels to show when the 'Previous'
        button is clicked

        """
        if self.pageTwo.IsShown():
            self.pageTwo.Hide()
            self.pageOne.Show()
            self.btnBack.Disable()
            self.btnNext.Enable()

        elif self.pageThree.IsShown():
            self.pageThree.Hide()
            self.pageTwo.Show()

        elif self.pageFour.IsShown():
            if self.pageTwo.locateBtn.IsEnabled():
                self.pageFour.Hide()
                self.pageTwo.Show()
            else:
                self.pageFour.Hide()
                self.pageThree.Show()

        elif self.pageFinish.IsShown():
            self.btnNext.SetLabel(_('Next >'))
            self.pageFinish.Hide()
            self.pageFour.Show()

        self.Layout()
    # -------------------------------------------------------------------#

    def wizard_Finish(self):
        """
        Get all settings and call `write_changes`
        to applies changes.

        """
        youtubedl = self.pageFour.ckbx_yn.GetValue()
        if not self.pageTwo.locateBtn.IsEnabled():
            binfound = 'local'
        elif not self.pageTwo.detectBtn.IsEnabled():
            binfound = 'system'

        write_changes(Wizard.getfileconf,
                      self.ffmpeg,
                      self.ffplay,
                      self.ffprobe,
                      youtubedl,
                      binfound
                      )
        self.Hide()
        wx.MessageBox(_("Re-start is required"),
                      _("Done!"), wx.ICON_INFORMATION, self)
        self.Destroy()



# -*- coding: UTF-8 -*-
"""
Name: filter_colorcorrection.py
Porpose: Show dialog to set color correction data based on FFmpeg syntax
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import wx
from videomass.vdms_utils.utils import time_to_integer
from videomass.vdms_utils.utils import integer_to_time
from videomass.vdms_utils.utils import clockset
from videomass.vdms_io.make_filelog import make_log_template
from videomass.vdms_threads.generic_task import FFmpegGenericTask


class ColorEQ(wx.Dialog):
    """
    A dialog tool to set video contrast, brightness,
    saturation, gamma values based on FFmpeg syntax.
    see docs at <https://ffmpeg.org/ffmpeg-filters.html#toc-eq>.
    See ``av_conversions.py`` -> ``on_Set_colorcorrect`` method
    for how to use this class.
    """
    get = wx.GetApp()
    OS = get.appset['ostype']
    LOGDIR = get.appset['logdir']
    TMPROOT = os.path.join(get.appset['cachedir'], 'tmp', 'ColorEQ')
    TMPSRC = os.path.join(TMPROOT, 'tmpsrc')
    TMPEDIT = os.path.join(TMPROOT, 'tmpedit')
    os.makedirs(TMPSRC, mode=0o777, exist_ok=True)
    os.makedirs(TMPEDIT, mode=0o777, exist_ok=True)
    # BACKGROUND = '#1b0413'

    def __init__(self, parent, colorset, iconreset, **kwa):
        """
        colorset: previus data already saved.
        width: source video width
        height: source video height
        fname: matches with the only one video file dragged
               into the list or the selected one if many.
        duration: Overall time duration of video.
        """
        self.filename = kwa['filename']
        name = os.path.splitext(os.path.basename(self.filename))[0]
        self.framesrc = os.path.join(ColorEQ.TMPSRC, f'{name}.png')
        self.frameedit = os.path.join(ColorEQ.TMPEDIT, f'{name}.png')
        self.fileclock = os.path.join(ColorEQ.TMPROOT, f'{name}.clock')
        # resizing values preserving aspect ratio for monitors
        thr = 150 if kwa['height'] > kwa['width'] else 270
        self.h_ratio = int((kwa['height'] / kwa['width']) * thr)
        self.w_ratio = int((kwa['width'] / kwa['height']) * self.h_ratio)
        self.contrast = ""
        self.brightness = ""
        self.saturation = ""
        self.gamma = ""
        tcheck = clockset(kwa['duration'], self.fileclock)
        self.clock = tcheck['duration']
        self.mills = tcheck['millis']

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        sizerBase = wx.BoxSizer(wx.VERTICAL)
        sizerpanels = wx.FlexGridSizer(2, 2, 2, 2)
        sizerBase.Add(sizerpanels, 0, wx.CENTER, 0)
        self.panel_img1 = wx.Panel(self, wx.ID_ANY,
                                   size=(self.w_ratio, self.h_ratio))
        sizerpanels.Add(self.panel_img1, 0, wx.ALL | wx.CENTER, 5)
        self.panel_img2 = wx.Panel(self, wx.ID_ANY,
                                   size=(self.w_ratio, self.h_ratio))
        sizerpanels.Add(self.panel_img2, 0, wx.ALL | wx.CENTER, 5)
        lab_imgsrc = wx.StaticText(self, wx.ID_ANY,
                                   label=_("Before"),
                                   style=wx.ST_NO_AUTORESIZE
                                   | wx.ALIGN_CENTRE_HORIZONTAL,
                                   )
        sizerpanels.Add(lab_imgsrc, 0, wx.CENTER | wx.EXPAND)
        lab_imgedit = wx.StaticText(self, wx.ID_ANY,
                                    label=_("After"),
                                    style=wx.ST_NO_AUTORESIZE
                                    | wx.ALIGN_CENTRE_HORIZONTAL,
                                    )
        sizerpanels.Add(lab_imgedit, 0, wx.CENTER | wx.EXPAND)
        sizerBase.Add(10, 10)
        msg = _("Search for a specific frame")
        stboxtime = wx.StaticBox(self, wx.ID_ANY, msg)
        sizertime = wx.StaticBoxSizer(stboxtime, wx.HORIZONTAL)
        sizerBase.Add(sizertime, 0, wx.ALL | wx.CENTER, 5)
        self.sld_time = wx.Slider(self, wx.ID_ANY,
                                  time_to_integer(self.clock),
                                  0,
                                  1 if not self.mills else self.mills,
                                  size=(250, -1),
                                  style=wx.SL_HORIZONTAL,
                                  )
        sizertime.Add(self.sld_time, 0, wx.ALL | wx.CENTER, 5)
        self.txttime = wx.StaticText(self, wx.ID_ANY, self.clock)
        sizertime.Add(self.txttime, 0, wx.ALL | wx.CENTER, 5)
        self.btn_load = wx.Button(self, wx.ID_ANY, _("Load"))
        self.btn_load.Disable()
        sizertime.Add(self.btn_load, 0, wx.ALL | wx.CENTER, 5)
        boxcolor = wx.StaticBox(self, wx.ID_ANY, (_("Color EQ")))
        sizercolor = wx.StaticBoxSizer(boxcolor, wx.VERTICAL)
        sizerBase.Add(sizercolor, 0, wx.ALL | wx.EXPAND, 5)
        sizerflex1 = wx.FlexGridSizer(0, 4, 0, 0)
        sizercolor.Add(sizerflex1, 0, wx.ALL | wx.CENTRE, 5)

        lbl_contrast = wx.StaticText(self, wx.ID_ANY, _('Contrast:'))
        sizerflex1.Add(lbl_contrast, 0, wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 0)
        self.sld_contrast = wx.Slider(self, wx.ID_ANY, 0, -100, 100,
                                      size=(200, -1), style=wx.SL_HORIZONTAL
                                      | wx.SL_AUTOTICKS
                                      | wx.SL_LABELS,
                                      )
        self.sld_contrast.SetToolTip(_("-100 to +100, default value 0"))
        sizerflex1.Add(self.sld_contrast, 0, wx.LEFT
                       | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 10)

        lbl_brigh = wx.StaticText(self, wx.ID_ANY, _('Brightness:'))
        sizerflex1.Add(lbl_brigh, 0, wx.LEFT
                       | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 20)
        self.sld_brightness = wx.Slider(self, wx.ID_ANY, 0, -100, 100,
                                        size=(200, -1), style=wx.SL_HORIZONTAL
                                        | wx.SL_AUTOTICKS
                                        | wx.SL_LABELS,
                                        )
        self.sld_brightness.SetToolTip(_("-100 to +100, default value 0"))
        sizerflex1.Add(self.sld_brightness, 0, wx.LEFT
                       | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 10)
        line = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                             size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                             name=wx.StaticLineNameStr,
                             )
        sizercolor.Add(line, 0, wx.ALL | wx.EXPAND, 5)
        sizerflex2 = wx.FlexGridSizer(0, 4, 0, 0)
        lbl_sat = wx.StaticText(self, wx.ID_ANY, _('Saturation:'))
        sizerflex2.Add(lbl_sat, 0, wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 0)
        self.sld_saturation = wx.Slider(self, wx.ID_ANY, 100, 0, 300,
                                        size=(200, -1), style=wx.SL_HORIZONTAL
                                        | wx.SL_AUTOTICKS
                                        | wx.SL_LABELS,
                                        )
        self.sld_saturation.SetToolTip(_("0 to 300, default value 100"))
        sizerflex2.Add(self.sld_saturation, 0, wx.LEFT
                       | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 10)
        lbl_gam = wx.StaticText(self, wx.ID_ANY, _('Gamma:'))
        sizerflex2.Add(lbl_gam, 0, wx.LEFT
                       | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 20)
        self.sld_gamma = wx.Slider(self, wx.ID_ANY, 10, 0, 100,
                                   size=(200, -1), style=wx.SL_HORIZONTAL
                                   | wx.SL_AUTOTICKS
                                   | wx.SL_LABELS,
                                   )
        self.sld_gamma.SetToolTip(_("0 to 100, default value 10"))
        sizerflex2.Add(self.sld_gamma, 0, wx.LEFT
                       | wx.ALIGN_CENTRE_VERTICAL
                       | wx.ALIGN_CENTRE_HORIZONTAL, 10)
        sizercolor.Add(sizerflex2, 0, wx.ALL | wx.CENTRE, 5)
        # bottom btns
        gridBtn = wx.GridSizer(1, 2, 0, 0)
        gridexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_reset = wx.Button(self, wx.ID_ANY, _("Reset"))
        btn_reset.SetBitmap(iconreset, wx.LEFT)
        gridBtn.Add(btn_reset, 0, wx.ALL, 5)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        gridexit.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        gridexit.Add(btn_ok, 0, wx.LEFT, 5)
        gridBtn.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        sizerBase.Add(gridBtn, 0, wx.EXPAND)
        self.SetSizer(sizerBase)
        sizerBase.Fit(self)
        self.Layout()
        # ----------------------Properties-----------------------#
        if ColorEQ.OS == 'Darwin':
            lab_imgsrc.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            lab_imgedit.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
        else:
            lab_imgsrc.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            lab_imgedit.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.SetTitle(_("Color Correction EQ Tool"))
        # ----------------------Binding (EVT)-------------------------#

        self.Bind(wx.EVT_COMMAND_SCROLL, self.on_seek_time, self.sld_time)
        self.Bind(wx.EVT_BUTTON, self.on_load_at_time, self.btn_load)

        if ColorEQ.OS == 'Windows':
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_contrast,
                      self.sld_contrast)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_contrast,
                      self.sld_contrast)
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_brightness,
                      self.sld_brightness)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_brightness,
                      self.sld_brightness)
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_saturation,
                      self.sld_saturation)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_saturation,
                      self.sld_saturation)
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_gamma,
                      self.sld_gamma)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_gamma, self.sld_gamma)

        if ColorEQ.OS == 'Darwin':
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_contrast,
                      self.sld_contrast)
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_brightness,
                      self.sld_brightness)
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_saturation,
                      self.sld_saturation)
            self.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.on_gamma,
                      self.sld_gamma)

        else:
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_contrast,
                      self.sld_contrast)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_brightness,
                      self.sld_brightness)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_saturation,
                      self.sld_saturation)
            self.Bind(wx.EVT_SCROLL_CHANGED, self.on_gamma, self.sld_gamma)

        self.Bind(wx.EVT_BUTTON, self.on_close, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)
        self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)

        if not self.mills:
            self.sld_time.Disable()

        if colorset:  # previus values
            self.set_default(colorset)
        self.process(self.framesrc)
        self.loader_initial_source()
        self.equalize_image(self.concat_filter())
    # -----------------------------------------------------------------------#

    def process(self, pathtosave, equalizer=''):
        """
        Generate a new frame at the clock position using
        ffmpeg `eq` filter.
        """
        logfile = make_log_template('generic_task.log',
                                    ColorEQ.LOGDIR,
                                    mode="w",
                                    )
        if not self.mills:
            sseg = ''
        else:
            sseg = f'-ss {self.clock}'
        eql = '' if not equalizer else f'-vf "{equalizer}"'
        arg = (f'{sseg} -i "{self.filename}" -f image2 '
               f'-update 1 -frames:v 1 {eql} "{pathtosave}"')
        thread = FFmpegGenericTask(arg, 'ColorEQ', logfile)
        thread.join()  # wait end thread
        error = thread.status
        if error:
            return error
        return None
    # -----------------------------------------------------------------------#

    def loader_initial_source(self):
        """
        Loads initial StaticBitmaps on panels 1 (source).
        """
        img = wx.Image(self.framesrc, wx.BITMAP_TYPE_ANY)
        img = img.Scale(self.w_ratio, self.h_ratio, wx.IMAGE_QUALITY_NORMAL)
        bitmap = img.ConvertToBitmap()
        wx.StaticBitmap(self.panel_img1, wx.ID_ANY, bitmap)
    # -----------------------------------------------------------------------#

    def loader_initial_edit(self):
        """
        Loads initial StaticBitmaps on panels 2 (edit)
        """
        img = wx.Image(self.frameedit, wx.BITMAP_TYPE_ANY)
        img = img.Scale(self.w_ratio, self.h_ratio, wx.IMAGE_QUALITY_NORMAL)
        bitmap = img.ConvertToBitmap()
        wx.StaticBitmap(self.panel_img2, wx.ID_ANY, bitmap)
    # -----------------------------------------------------------------------#

    def set_default(self, colorset):
        """
        Set previus setting stored on `colorset` data.
        """
        filterlist = colorset.split(':')
        filterlist[0] = filterlist[0].split(sep='=', maxsplit=1)[1]
        eqval = dict(x.split('=') for x in filterlist)
        # set attributes strings if eqval has keys else as init ("")
        for key, val in eqval.items():
            if key == 'contrast':
                self.contrast = f'contrast={val}'
            if key == 'brightness':
                self.brightness = f'brightness={val}'
            if key == 'saturation':
                self.saturation = f'saturation={val}'
            if key == 'gamma':
                self.gamma = f'gamma={val}'

        # compute to sliders values (int)
        contrast = round((float(eqval.get('contrast', 1.0)) * 100 - 100) / 2)
        brightness = int(float(eqval.get('brightness', 0.0)) * 100)
        saturation = int(float(eqval.get('saturation', 1.0)) * 100)
        gamma = int(float(eqval.get('gamma', 1.0)) * 10)
        # set sliders
        self.sld_contrast.SetValue(contrast)
        self.sld_brightness.SetValue(brightness)
        self.sld_saturation.SetValue(saturation)
        self.sld_gamma.SetValue(gamma)
    # -----------------------------------------------------------------------#

    def equalize_image(self, equalizer=''):
        """
        Sends the equalization values to the process
        """
        error = self.process(self.frameedit, equalizer=equalizer)
        if error:
            wx.MessageBox(f'{error}', 'ERROR', wx.ICON_ERROR, self)
            return
        self.loader_initial_edit()
    # -----------------------------------------------------------------------#

    def concat_filter(self):
        """
        Concatenates all EQ values.
        Returns the EQ filter string in ffmpeg syntax.
        """
        orderf = (self.contrast, self.brightness, self.saturation, self.gamma)
        concat = ''.join([f'{x}:' for x in orderf if x])[:-1]
        if not concat:
            return concat
        return f'eq={concat}'
    # -----------------------------------------------------------------------#

    def on_seek_time(self, event):
        """
        Event to get slider position.
        Getting the value in ms from the time slider,
        converts it to clock format e.g (00:00:00),
        and sets the label with the converted value.
        """
        seek = self.sld_time.GetValue()
        clock = integer_to_time(seek, False)  # to 24-hour
        self.txttime.SetLabel(clock)  # update StaticText
        if not self.btn_load.IsEnabled():
            self.btn_load.Enable()
    # -----------------------------------------------------------------------#

    def on_load_at_time(self, event):
        """
        Reloads all images frame at a given time clock point
        """
        seek = self.sld_time.GetValue()
        self.clock = integer_to_time(seek, False)  # to 24-hour
        error = self.process(self.framesrc)
        if error:
            wx.MessageBox(f'{error}', 'ERROR', wx.ICON_ERROR, self)
            return
        self.loader_initial_source()

        error = self.process(self.frameedit, self.concat_filter())
        if error:
            wx.MessageBox(f'{error}', 'ERROR', wx.ICON_ERROR, self)
            return
        self.loader_initial_edit()

        with open(self.fileclock, "w", encoding='utf8') as atime:
            atime.write(self.clock)
        self.btn_load.Disable()
    # -----------------------------------------------------------------------#

    def on_contrast(self, event):
        """
        Scroll event for contrast EQ.
        Set the contrast expression. The value must be a float
        value in range -1000.0 to 1000.0. The default value is "1".
        """
        val = 1.0 + self.sld_contrast.GetValue() / 50
        self.contrast = "" if val == 1.0 else f'contrast={val}'
        self.equalize_image(self.concat_filter())
    # -----------------------------------------------------------------------#

    def on_brightness(self, event):
        """
        Scroll event for brightness EQ.
        Set the brightness expression. The value must be a float
        value in range -1.0 to 1.0. The default value is "0".
        """
        val = 0.0 + self.sld_brightness.GetValue() / 100
        self.brightness = "" if val == 0.0 else f'brightness={val}'
        self.equalize_image(self.concat_filter())
    # -----------------------------------------------------------------------#

    def on_saturation(self, event):
        """
        Scroll event for saturation EQ.
        Set the saturation expression. The value must be a float
        in range 0.0 to 3.0. The default value is "1".
        """
        val = 0.0 + self.sld_saturation.GetValue() / 100
        self.saturation = "" if val == 1.0 else f'saturation={val}'
        self.equalize_image(self.concat_filter())
    # -----------------------------------------------------------------------#

    def on_gamma(self, event):
        """
        Scroll event for gamma EQ.
        Set the gamma expression. The value must be a float
        in range 0.1 to 10.0. The default value is "1".
        """
        val = self.sld_gamma.GetValue() / 10
        self.gamma = "" if val == 1.0 else f'gamma={val}'
        self.equalize_image(self.concat_filter())
    # -----------------------------------------------------------------------#

    def on_reset(self, event):
        """
        Reset all control values
        """
        self.contrast = ""
        self.brightness = ""
        self.saturation = ""
        self.gamma = ""
        self.sld_contrast.SetValue(0)
        self.sld_brightness.SetValue(0)
        self.sld_saturation.SetValue(100)
        self.sld_gamma.SetValue(10)
        self.equalize_image(self.concat_filter())
    # -----------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything.
        Don't use self.Destroy() here, it is used by the caller
        """
        event.Skip()
    # -----------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Before destroying the dialog getvalue() will be called.
        Don't use self.Destroy() here, it is used by the caller
        """
        event.Skip()
    # -----------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller. See the caller for more info and usage.
        """
        return self.concat_filter()



# -*- coding: UTF-8 -*-
"""
Name: generic_task.py
Porpose: Execute a generic task with FFmpeg
Compatibility: Python3 (Unix, Windows)
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.04.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from threading import Thread
import platform
import subprocess
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
if not platform.system() == 'Windows':
    import shlex


class FFmpegGenericTask(Thread):
    """
    Run a generic task with FFmpeg as a separate thread.
    This class does not redirect any progress output information
    for debugging, however you can get the exit status message.

    USAGE:
        >>> thread = FFmpegGenericTask(args)
        >>> thread.join()
        >>> error = thread.status
        >>> if error:
        >>>     error

    Raise: `OSError` if not FFmpeg
    Return: None

    """
    get = wx.GetApp()
    appdata = get.appset

    def __init__(self, args, procname='Unknown', logfile='logfile.log'):
        """
        Attributes defined here:

        self.args, a string containing the command args
        of FFmpeg, excluding the command itself `ffmpeg`

        self.status, If the exit status is true (which can be an
        exception or error message given by returncode) it must be
        handled appropriately, in the other case it is None.

        """
        self.logfile = logfile
        self.procname = procname
        self.args = args
        self.status = None

        Thread.__init__(self)
        self.start()
    # ----------------------------------------------------------------#

    def run(self):
        """
        Get and redirect output errors on p.returncode instance and
        OSError exception. Otherwise the getted output is None

        """
        cmd = (f'"{FFmpegGenericTask.appdata["ffmpeg_cmd"]}" '
               f'{FFmpegGenericTask.appdata["ffmpeg_default_args"]} '
               f'{self.args}'
               )
        self.logwrite(f'From: {self.procname}\n{cmd}\n')

        if not platform.system() == 'Windows':
            cmd = shlex.split(cmd)
        try:
            with Popen(cmd,
                       stderr=subprocess.PIPE,
                       universal_newlines=True,
                       encoding='utf8',
                       ) as proc:
                error = proc.communicate()
                self.logwrite(f'[FFMPEG]:\n{error}')
                if proc.returncode:  # ffmpeg error
                    if error[1]:
                        self.status = error[1]
                    else:
                        self.status = "FFmpeg Unrecognized error"

        except OSError as err:  # command not found
            self.status = err

        if self.status:
            self.logerror()
        wx.CallAfter(pub.sendMessage,
                     "RESULT_EVT",
                     status=''
                     )
    # ----------------------------------------------------------------#

    def logwrite(self, cmd):
        """
        write ffmpeg command log
        """
        with open(self.logfile, "a", encoding='utf8') as log:
            log.write(f"{cmd}\n")
    # ----------------------------------------------------------------#

    def logerror(self):
        """
        write ffmpeg volumedected errors
        """
        with open(self.logfile, "a", encoding='utf8') as logerr:
            logerr.write(f"\n[FFMPEG] generic_task "
                         f"ERRORS:\n{self.status}\n")



# -*- coding: UTF-8 -*-
"""
Name: volumedetect.py
Porpose: Audio Peak level volume analyzes
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.17.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from threading import Thread
import subprocess
import platform
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.make_filelog import make_log_template
if not platform.system() == 'Windows':
    import shlex


class VolumeDetectThread(Thread):
    """
    This class represents a separate subprocess thread to get
    audio volume peak level when required for audio normalization
    process.

    NOTE: all error handling (including verification of the
    existence of files) is entrusted to ffmpeg, except for the
    lack of ffmpeg of course.

    """

    def __init__(self, timeseq, filelist, audiomap, logdir, ffmpeg_url):
        """
        Replace /dev/null with NUL on Windows.

        self.status: None, if nothing error,
                     'str error' if errors.
        self.data: it is a tuple containing the list of audio volume
                   parameters and the self.status of the output error,
                   in the form:
                   ([[maxvol, medvol], [etc,etc]], None or "str errors")
        """
        self.filelist = filelist
        self.time_seq = timeseq
        self.audiomap = audiomap
        self.ffmpeg_url = ffmpeg_url
        self.status = None
        self.data = None
        self.nul = 'NUL' if platform.system() == 'Windows' else '/dev/null'
        self.logf = os.path.join(logdir, 'volumedected.log')
        make_log_template('volumedected.log', logdir, mode="w")
        # set initial file LOG

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())
    # ----------------------------------------------------------------#

    def run(self):
        """
        Audio volume data is getted by the thread's caller using
        the thread.data method (see io_tools).
        NOTE: wx.callafter(pub...) do not send data to pop-up
              dialog, but a empty string that is useful to get
              the end of the process to close of the pop-up

        """
        volume = []

        for files in self.filelist:
            cmd = (f'"{self.ffmpeg_url}" -hide_banner '
                   f'{self.time_seq[0]} '
                   f'-i "{files}" '
                   f'{self.time_seq[1]} '
                   f'{self.audiomap} '
                   f'-af volumedetect -vn -sn -dn -f null '
                   f'{self.nul}'
                   )
            self.logwrite(cmd)

            if not platform.system() == 'Windows':
                cmd = shlex.split(cmd)
            try:
                with Popen(cmd,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT,
                           universal_newlines=True,
                           encoding='utf8',
                           ) as proc:

                    output = proc.communicate()

                    if proc.returncode:  # if error occurred
                        self.status = output[0]
                        break

                    raw_list = output[0].split()  # splitta tutti gli spazi
                    if 'mean_volume:' in raw_list:
                        mean_volume = raw_list.index("mean_volume:")
                        # mean_volume is indx integear
                        medvol = f"{raw_list[mean_volume + 1]} dB"
                        max_volume = raw_list.index("max_volume:")
                        # max_volume is indx integear
                        maxvol = f"{raw_list[max_volume + 1]} dB"
                        volume.append([maxvol, medvol])

            except (OSError, FileNotFoundError) as err:  # ffmpeg do not exist
                self.status = err
                break

        self.data = (volume, self.status)

        if self.status:
            self.logerror()

        wx.CallAfter(pub.sendMessage,
                     "RESULT_EVT",
                     status=''
                     )
    # ----------------------------------------------------------------#

    def logwrite(self, cmd):
        """
        write ffmpeg command log
        """
        with open(self.logf, "a", encoding='utf8') as log:
            log.write(f"{cmd}\n")
    # ----------------------------------------------------------------#

    def logerror(self):
        """
        write ffmpeg volumedected errors
        """
        with open(self.logf, "a", encoding='utf8') as logerr:
            logerr.write(f"\n[FFMPEG] volumedetect "
                         f"ERRORS:\n{self.status}\n")



# -*- coding: UTF-8 -*-
"""
Name: ffmpeg_formats.py
Porpose: Show the available formats on the FFmpeg
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.13.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
from pubsub import pub


class FFmpegFormats(wx.Dialog):
    """
    It shows a dialog box with a pretty kind of GUI to view
    the formats available on FFmpeg

    """
    def __init__(self, dict_formats, OS):
        """
        with 'None' not depend from parent:
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)

        With parent, -1:
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        if close videomass also close parent window:

        """
        get = wx.GetApp()  # get data from bootstrap
        vidicon = get.iconset['videomass']
        wx.Dialog.__init__(self, None,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           | wx.DIALOG_NO_PARENT
                           )
        # add panel
        self.panel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL
                              | wx.BORDER_THEME,
                              )
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        notebook = wx.Notebook(self.panel, wx.ID_ANY)
        sizer_base.Add(notebook, 1, wx.ALL | wx.EXPAND, 5)
        # ----- nb1
        notebook_pane_1 = wx.Panel(notebook, wx.ID_ANY)
        dmx = wx.ListCtrl(notebook_pane_1,
                          wx.ID_ANY,
                          style=wx.LC_REPORT
                          | wx.SUNKEN_BORDER,
                          )
        sizer_tab1 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1.Add(dmx, 1, wx.ALL | wx.EXPAND, 5)
        notebook_pane_1.SetSizer(sizer_tab1)
        notebook.AddPage(notebook_pane_1, (_("Demuxing only")))
        # ----- nb2
        notebook_pane_2 = wx.Panel(notebook, wx.ID_ANY)
        mx = wx.ListCtrl(notebook_pane_2,
                         wx.ID_ANY,
                         style=wx.LC_REPORT
                         | wx.SUNKEN_BORDER,
                         )
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab2.Add(mx, 1, wx.ALL | wx.EXPAND, 5)
        notebook_pane_2.SetSizer(sizer_tab2)
        notebook.AddPage(notebook_pane_2, (_("Muxing only")))
        # ----- nb3
        notebook_pane_3 = wx.Panel(notebook, wx.ID_ANY)
        dmx_mx = wx.ListCtrl(notebook_pane_3,
                             wx.ID_ANY,
                             style=wx.LC_REPORT
                             | wx.SUNKEN_BORDER,
                             )
        sizer_tab3 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab3.Add(dmx_mx, 1, wx.ALL | wx.EXPAND, 5)
        notebook_pane_3.SetSizer(sizer_tab3)
        notebook.AddPage(notebook_pane_3, (_("Demuxing/Muxing support")))

        # ----- btns
        button_close = wx.Button(self.panel, wx.ID_CLOSE, "")
        grid_buttons = wx.GridSizer(1, 1, 0, 0)
        grid_buttons.Add(button_close, 1, wx.ALL, 5)
        sizer_base.Add(grid_buttons, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)

        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(vidicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetTitle(_("FFmpeg file formats"))
        self.SetMinSize((500, 400))
        dmx.SetMinSize((500, 400))
        self.panel.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()
        # ----------------------Properties----------------------#
        dmx.InsertColumn(0, _('supported formats'), width=180)
        dmx.InsertColumn(1, _('description'), width=450)
        # mx.SetMinSize((500, 400))
        mx.InsertColumn(0, _('supported formats'), width=180)
        mx.InsertColumn(1, _('description'), width=450)
        # dmx_mx.SetMinSize((500, 400))
        dmx_mx.InsertColumn(0, _('supported formats'), width=180)
        dmx_mx.InsertColumn(1, _('description'), width=450)

        if OS == 'Darwin':
            dmx.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            mx.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            dmx_mx.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
        else:
            dmx.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            mx.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            dmx_mx.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))

        # delete previous append:
        dmx.DeleteAllItems()
        mx.DeleteAllItems()
        dmx_mx.DeleteAllItems()

        # populate dmx listctrl output:
        index = 0
        ds = dict_formats['Demuxing Supported']
        if ds:
            for a in ds:
                s = " ".join(a.split()).split(None, 1)
                if len(s) == 1:
                    key, value = s[0], ''
                else:
                    key, value = s[0], s[1]
                dmx.InsertItem(index, key)
                dmx.SetItem(index, 1, value)
                index = 0

        # populate mx listctrl output:
        index = 0
        ms = dict_formats['Muxing Supported']
        if ms:
            for a in ms:
                s = " ".join(a.split()).split(None, 1)
                if len(s) == 1:
                    key, value = s[0], ''
                else:
                    key, value = s[0], s[1]
                mx.InsertItem(index, key)
                mx.SetItem(index, 1, value)
                index = 0

        # populate dmx_mx listctrl output:
        index = 0
        mds = dict_formats["Mux/Demux Supported"]
        if mds:
            for a in mds:
                s = " ".join(a.split()).split(None, 1)
                if len(s) == 1:
                    key, value = s[0], ''
                else:
                    key, value = s[0], s[1]
                dmx_mx.InsertItem(index, key)
                dmx_mx.SetItem(index, 1, value)
                index = 0

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)  # controlla la chiusura (x)

    # ----------------------Event handler (callback)----------------------#

    def on_close(self, event):
        """
        Destroy this window
        """
        pub.sendMessage("DESTROY_ORPHANED_WINDOWS", msg='FFmpegFormats')



# -*- coding: UTF-8 -*-
"""
Name: ydl_downloader.py
Porpose: long processing task using yt_dlp python library
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.07.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from threading import Thread
import itertools
import wx
from pubsub import pub
from videomass.vdms_io.make_filelog import logwrite
if wx.GetApp().appset['use-downloader']:
    import yt_dlp


class MyLogger:
    """
    Intercepts youtube-dl's output by setting a logger object;
    * Log messages to a logging.Logger instance.
    <https://github.com/ytdl-org/youtube-dl/tree/3e4cedf9e8cd315
    7df2457df7274d0c842421945#embedding-youtube-dl>
    """

    def __init__(self):
        """
        define instace attributes
        """
        self.msg = None

    def debug(self, msg):
        """
        Get debug messages. Note, both debug and info
        are passed into debug. You can distinguish them
        by the prefix '[debug] '
        """
        wx.CallAfter(pub.sendMessage,
                     "UPDATE_YDL_EVT",
                     output=msg,
                     duration='',
                     status='DEBUG',
                     )
        self.msg = msg

    def warning(self, msg):
        """
        Get warning messages
        """
        msg = f'WARNING: {msg}'
        wx.CallAfter(pub.sendMessage,
                     "UPDATE_YDL_EVT",
                     output=msg,
                     duration='',
                     status='WARNING',
                     )

    def error(self, msg):
        """
        Get error messages
        """
        wx.CallAfter(pub.sendMessage,
                     "UPDATE_YDL_EVT",
                     output=msg,
                     duration='',
                     status='ERROR',
                     )
# -------------------------------------------------------------------------#


def my_hook(data):
    """
    progress_hooks is A list of functions that get called on
    download progress. See  `help(youtube_dl.YoutubeDL)`
    """
    if data['status'] == 'downloading':
        keys = ('_percent_str', '_total_bytes_str', '_speed_str', '_eta_str')

        wx.CallAfter(pub.sendMessage,
                     "UPDATE_YDL_EVT",
                     output='',
                     duration={x: data.get(x, 'N/A') for x in keys},
                     status='DOWNLOAD',
                     )
    if data['status'] == 'finished':
        wx.CallAfter(pub.sendMessage,
                     "COUNT_YTDL_EVT",
                     count='',
                     fsource='',
                     destination='',
                     duration='',
                     end='ok',
                     )
        wx.CallAfter(pub.sendMessage,
                     "UPDATE_YDL_EVT",
                     output='',
                     duration='Done downloading, now converting ...',
                     status='FINISHED',
                     )
# -------------------------------------------------------------------------#


class YdlDownloader(Thread):
    """
    Embed youtube-dl as module into a separated thread in order
    to get output in real time during downloading and conversion .
    For a list of available options see:

    <https://github.com/ytdl-org/youtube-dl/blob/master/youtube_dl/YoutubeDL.py#L129-L279>
    <https://github.com/ytdl-org/youtube-dl/blob/master/youtube_dl/options.py>

    or by help(youtube_dl.YoutubeDL)

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    appdata = get.appset
    FFMPEG_URL = appdata['ffmpeg_cmd']

    if appdata['playlistsubfolder']:
        SUBDIR = '%(uploader)s/%(playlist_title)s/%(playlist_index)s - '
    else:
        SUBDIR = ''

    def __init__(self, varargs, logname):
        """
        Attributes defined here:
        self.stop_work_thread:  process terminate value
        self.args['urls']:          urls list
        self.opt:           option dict data type to adding
        self.args['outdir']:     pathname destination
        self.args['code']:          Format Code, else empty string ''
        self.count:         increases progressive account elements
        self.args['countmax']:      length of urls items list
        self.args['logname']:       file name to log messages for logging
        """
        self.stop_work_thread = False  # process terminate
        self.opt = varargs[4]
        self.count = 0
        self.args = {'urls': varargs[1],
                     'code': varargs[6],
                     'outdir': varargs[3],
                     'logname': logname,
                     'countmax': len(varargs[1]),
                     }
        Thread.__init__(self)
        self.start()  # run()

    def run(self):
        """
        Apply the options passed by the user for the
        download process with yt_dlp

        """
        for url, code in itertools.zip_longest(self.args['urls'],
                                               self.args['code'],
                                               fillvalue='',
                                               ):
            if '/playlist' in url or not self.opt['noplaylist']:
                outtmpl = YdlDownloader.SUBDIR + self.opt['outtmpl']
            else:
                outtmpl = self.opt['outtmpl']

            format_code = code if code else self.opt['format']
            self.count += 1
            count = f"URL {self.count}/{self.args['countmax']}"

            wx.CallAfter(pub.sendMessage,
                         "COUNT_YTDL_EVT",
                         count=count,
                         fsource=f'Source: {url}',
                         destination='',
                         duration=100,
                         end='',
                         )

            if self.stop_work_thread:
                break

            ydl_opts = {
                'compat_opts': 'youtube-dl',
                'external_downloader': self.appdata["external_downloader"],
                'external_downloader_args':
                    self.appdata["external_downloader_args"],
                'format': format_code,
                'extractaudio': self.opt['format'],
                'outtmpl': f"{self.args['outdir']}/{outtmpl}",
                'writesubtitles': self.opt['writesubtitles'],
                'subtitleslangs': self.opt['subtitleslangs'],
                'writeautomaticsub': self.opt['writeautomaticsub'],
                'skip_download': self.opt['skip_download'],
                'addmetadata': self.opt['addmetadata'],
                'restrictfilenames': self.opt['restrictfilenames'],
                'ignoreerrors': True,
                'no_warnings': False,
                'writethumbnail': self.opt['writethumbnail'],
                'noplaylist': self.opt['noplaylist'],
                'playlist_items': self.opt['playlist_items'].get(url, None),
                'overwrites': self.opt['overwrites'],
                'no_color': True,
                'nocheckcertificate': self.opt['nocheckcertificate'],
                'ffmpeg_location': f'{YdlDownloader.FFMPEG_URL}',
                'postprocessors': self.opt['postprocessors'],
                'logger': MyLogger(),
                'progress_hooks': [my_hook],
            }
            logtxt = f'{count}\n{ydl_opts}'
            logwrite(logtxt, '', self.args['logname'])  # write log cmd

            with yt_dlp.YoutubeDL(ydl_opts) as ydl:
                ydl.download([f"{url}"])

        wx.CallAfter(pub.sendMessage, "END_YTDL_EVT")

    def stop(self):
        """
        Sets the stop work thread to terminate the current process
        """
        self.stop_work_thread = True
